@@ -93,8 +93,9 @@ typedef enum _NcDataSNIACovData
   NC_DATA_SNIA_COV_VAR_COLOUR,
   NC_DATA_SNIA_COV_VAR_MAG_WIDTH,
   NC_DATA_SNIA_COV_VAR_MAG_COLOUR,
-  NC_DATA_SNIA_COV_VAR_WIDTH_COLOUR, /*< private >*/
-  NC_DATA_SNIA_COV_TOTAL_LENGTH,     /*< skip >*/
+  NC_DATA_SNIA_COV_VAR_WIDTH_COLOUR,
+  /* < private > */
+  NC_DATA_SNIA_COV_TOTAL_LENGTH, /*< skip >*/
 } NcDataSNIACovData;
 
 #define NC_DATA_SNIA_COV_LENGTH NC_DATA_SNIA_COV_ABSMAG_SET
@@ -136,8 +137,9 @@ typedef enum _NcDataSNIACovDataV1
   NC_DATA_SNIA_COV_V1_MAG_COLOUR,
   NC_DATA_SNIA_COV_V1_WIDTH_WIDTH,
   NC_DATA_SNIA_COV_V1_WIDTH_COLOUR,
-  NC_DATA_SNIA_COV_V1_COLOUR_COLOUR, /*< private >*/
-  NC_DATA_SNIA_COV_V1_TOTAL_LENGTH,  /*< skip >*/
+  NC_DATA_SNIA_COV_V1_COLOUR_COLOUR,
+  /* < private > */
+  NC_DATA_SNIA_COV_V1_TOTAL_LENGTH, /*< skip >*/
 } NcDataSNIACovDataV1;
 
 #define NC_DATA_SNIA_COV_V1_LENGTH NC_DATA_SNIA_COV_V1_ABSMAG_SET
@@ -199,8 +201,9 @@ typedef enum _NcDataSNIACovOrder
   NC_DATA_SNIA_COV_ORDER_MAG_COLOUR,
   NC_DATA_SNIA_COV_ORDER_WIDTH_WIDTH,
   NC_DATA_SNIA_COV_ORDER_WIDTH_COLOUR,
-  NC_DATA_SNIA_COV_ORDER_COLOUR_COLOUR, /*< private >*/
-  NC_DATA_SNIA_COV_ORDER_LENGTH,        /*< skip >*/
+  NC_DATA_SNIA_COV_ORDER_COLOUR_COLOUR,
+  /* < private > */
+  NC_DATA_SNIA_COV_ORDER_LENGTH, /*< skip >*/
 } NcDataSNIACovOrder;
 
 struct _NcDataSNIACovClass
