@@ -53,7 +53,8 @@ typedef struct _NcClusterMassLnnormal NcClusterMassLnnormal;
 typedef enum _NcClusterMassLnnormalParams
 {
   NC_CLUSTER_MASS_LNNORMAL_BIAS = 0,
-  NC_CLUSTER_MASS_LNNORMAL_SIGMA,  /*< private >*/
+  NC_CLUSTER_MASS_LNNORMAL_SIGMA,  
+  /* < private > */
   NC_CLUSTER_MASS_LNNORMAL_SPARAM_LEN, /*< skip >*/
 } NcClusterMassLnnormalParams;
 
