@@ -38,7 +38,7 @@
 
 #include "lss/nc_cluster_mass_vanderlinde.h"
 #include "math/integral.h"
-#include "math/memory_pool.h"
+#include "math/ncm_memory_pool.h"
 #include "math/ncm_cfg.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
