@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*-  */
 /***************************************************************************
  *            ncm_diff.c
  *
@@ -70,7 +71,7 @@ enum
   PROP_SIZE,
 };
 
-G_DEFINE_TYPE (NcmDiff, ncm_diff, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (NcmDiff, ncm_diff, G_TYPE_OBJECT);
 
 static void
 ncm_diff_init (NcmDiff *diff)
@@ -182,8 +183,6 @@ ncm_diff_class_init (NcmDiffClass *klass)
 {
   GObjectClass* object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (NcmDiffPrivate));
-
   object_class->set_property = &_ncm_diff_set_property;
   object_class->get_property = &_ncm_diff_get_property;
   object_class->constructed  = &_ncm_diff_constructed;
