@@ -56,13 +56,8 @@ G_BEGIN_DECLS
 #define NCM_IS_FIT_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), NCM_TYPE_FIT))
 #define NCM_FIT_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NCM_TYPE_FIT, NcmFitClass))
 
-#define NCM_TYPE_FIT_CONSTRAINT    (ncm_fit_constraint_get_type ())
-#define NCM_FIT_CONSTRAINT(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), NCM_TYPE_FIT_CONSTRAINT, NcmFitConstraint))
-#define NCM_IS_FIT_CONSTRAINT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NCM_TYPE_FIT_CONSTRAINT))
-
 typedef struct _NcmFitClass NcmFitClass;
 typedef struct _NcmFit NcmFit;
-typedef struct _NcmFitConstraint NcmFitConstraint;
 
 /**
  * NcmFitType:
@@ -161,26 +156,15 @@ struct _NcmFit
   gdouble m2lnL_abstol;
   gdouble params_reltol;
   GTimer *timer;
-  GPtrArray *equality_constraints;
-  GPtrArray *inequality_constraints;
+  NcmObjArray *equality_constraints;
+	GArray *equality_constraints_tot;
+  NcmObjArray *inequality_constraints;
+	GArray *inequality_constraints_tot;
   NcmFit *sub_fit;
   NcmDiff *diff;
 };
 
-struct _NcmFitConstraint
-{
-  /*< private >*/
-  NcmFit *fit;
-  NcmMSetFunc *func;
-  gdouble tot;
-};
-
 GType ncm_fit_get_type (void) G_GNUC_CONST;
-GType ncm_fit_constraint_get_type (void) G_GNUC_CONST;
-
-NcmFitConstraint *ncm_fit_constraint_new (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
-NcmFitConstraint *ncm_fit_constraint_dup (NcmFitConstraint *fitc);
-void ncm_fit_constraint_free (NcmFitConstraint *fitc);
 
 NcmFit *ncm_fit_new (NcmFitType ftype, gchar *algo_name, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
 NcmFit *ncm_fit_ref (NcmFit *fit);
@@ -203,6 +187,8 @@ gdouble ncm_fit_get_m2lnL_reltol (NcmFit *fit);
 gdouble ncm_fit_get_m2lnL_abstol (NcmFit *fit);
 gdouble ncm_fit_get_params_reltol (NcmFit *fit);
 
+NcmMSet *ncm_fit_peek_mset (NcmFit *fit);
+
 G_INLINE_FUNC void ncm_fit_params_set (NcmFit *fit, guint i, const gdouble x);
 G_INLINE_FUNC void ncm_fit_params_set_vector (NcmFit *fit, NcmVector *x);
 G_INLINE_FUNC void ncm_fit_params_set_vector_offset (NcmFit *fit, NcmVector *x, guint offset);
@@ -210,8 +196,8 @@ G_INLINE_FUNC void ncm_fit_params_set_array (NcmFit *fit, const gdouble *x);
 G_INLINE_FUNC void ncm_fit_params_set_gsl_vector (NcmFit *fit, const gsl_vector *x);
 G_INLINE_FUNC void ncm_fit_params_update (NcmFit *fit);
 
-void ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
-void ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
+void ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, const gdouble tot);
+void ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, const gdouble tot);
 void ncm_fit_remove_equality_constraints (NcmFit *fit);
 void ncm_fit_remove_inequality_constraints (NcmFit *fit);
 guint ncm_fit_has_equality_constraints (NcmFit *fit);
