@@ -38,15 +38,11 @@
 #include "build_cfg.h"
 
 #include "math/ncm_matrix.h"
+#include "math/ncm_vector.h"
 #include "math/ncm_lapack.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
-#if (defined HAVE_CLAPACK_H) && (defined HAVE_CLAPACK_DPOTRF)
-#include <clapack.h>
-#else
-#include <gsl/gsl_cblas.h>
 #include <gsl/gsl_linalg.h>
-#endif
 #endif /* NUMCOSMO_GIR_SCAN */
 
 enum
@@ -443,6 +439,20 @@ ncm_matrix_const_new_variant (GVariant *var)
   }
 }
 
+/**
+ * ncm_matrix_ref:
+ * @cm: a #NcmMatrix
+ *
+ * Increase the reference count of @cm by one.
+ *
+ * Returns: (transfer full): @cm
+ */
+NcmMatrix *
+ncm_matrix_ref (NcmMatrix *cm)
+{
+  return g_object_ref (cm);
+}
+
 /**
  * ncm_matrix_get_submatrix:
  * @cm: a #NcmMatrix
@@ -461,6 +471,10 @@ NcmMatrix *
 ncm_matrix_get_submatrix (NcmMatrix *cm, guint k1, guint k2, guint nrows, guint ncols)
 {
   NcmMatrix *scm = g_object_new (NCM_TYPE_MATRIX, NULL);
+
+	g_assert_cmpuint (nrows + k1, <=, ncm_matrix_nrows (cm));
+	g_assert_cmpuint (ncols + k2, <=, ncm_matrix_ncols (cm));
+	
   scm->mv = gsl_matrix_submatrix (ncm_matrix_gsl (cm), k1, k2, nrows, ncols);
 
   scm->pdata = g_object_ref (cm);
@@ -948,7 +962,7 @@ ncm_matrix_copy_triangle (NcmMatrix *cm, gchar UL)
       }
     }
   }
-  else
+  else if (UL == 'L')
   {
     for (i = 0; i < nrows; i++)
     {
@@ -958,6 +972,8 @@ ncm_matrix_copy_triangle (NcmMatrix *cm, gchar UL)
       }
     }
   }
+  else
+    g_assert_not_reached ();
 }
 
 /**
@@ -989,7 +1005,7 @@ ncm_matrix_dsymm (NcmMatrix *cm, gchar UL, const gdouble alpha, NcmMatrix *A, Nc
                ncm_matrix_data (cm), ncm_matrix_tda (cm));
 }
 
-enum CBLAS_ORDER
+CBLAS_TRANSPOSE
 _ncm_matrix_check_trans (const gchar *func_name, gchar T)
 {
   switch (T)
@@ -1022,12 +1038,12 @@ _ncm_matrix_check_trans (const gchar *func_name, gchar T)
 void 
 ncm_matrix_dgemm (NcmMatrix *cm, gchar TransA, gchar TransB, const gdouble alpha, NcmMatrix *A, NcmMatrix *B, const gdouble beta)
 {
-  enum CBLAS_TRANSPOSE cblas_TransA = _ncm_matrix_check_trans ("ncm_matrix_dgemm", TransA);
-  enum CBLAS_TRANSPOSE cblas_TransB = _ncm_matrix_check_trans ("ncm_matrix_dgemm", TransB);
-  const gsize opA_nrows             = (cblas_TransA == CblasNoTrans) ? ncm_matrix_nrows (A) : ncm_matrix_ncols (A);
-  const gsize opA_ncols             = (cblas_TransA == CblasNoTrans) ? ncm_matrix_ncols (A) : ncm_matrix_nrows (A);
-  const gsize opB_nrows             = (cblas_TransB == CblasNoTrans) ? ncm_matrix_nrows (B) : ncm_matrix_ncols (B);
-  const gsize opB_ncols             = (cblas_TransB == CblasNoTrans) ? ncm_matrix_ncols (B) : ncm_matrix_nrows (B);
+  CBLAS_TRANSPOSE cblas_TransA = _ncm_matrix_check_trans ("ncm_matrix_dgemm", TransA);
+  CBLAS_TRANSPOSE cblas_TransB = _ncm_matrix_check_trans ("ncm_matrix_dgemm", TransB);
+  const gsize opA_nrows        = (cblas_TransA == CblasNoTrans) ? ncm_matrix_nrows (A) : ncm_matrix_ncols (A);
+  const gsize opA_ncols        = (cblas_TransA == CblasNoTrans) ? ncm_matrix_ncols (A) : ncm_matrix_nrows (A);
+  const gsize opB_nrows        = (cblas_TransB == CblasNoTrans) ? ncm_matrix_nrows (B) : ncm_matrix_ncols (B);
+  const gsize opB_ncols        = (cblas_TransB == CblasNoTrans) ? ncm_matrix_ncols (B) : ncm_matrix_nrows (B);
 
   g_assert_cmpuint (opA_ncols, ==, opB_nrows);
 
@@ -1054,7 +1070,7 @@ ncm_matrix_dgemm (NcmMatrix *cm, gchar TransA, gchar TransB, const gdouble alpha
 gint
 ncm_matrix_cholesky_decomp (NcmMatrix *cm, gchar UL)
 {
-  gint ret = ncm_lapack_dpotrf (UL, ncm_matrix_nrows (cm), ncm_matrix_data (cm), ncm_matrix_nrows (cm));
+  gint ret = ncm_lapack_dpotrf (UL, ncm_matrix_nrows (cm), ncm_matrix_data (cm), ncm_matrix_tda (cm));
   return ret;
 }
 
@@ -1070,7 +1086,7 @@ ncm_matrix_cholesky_decomp (NcmMatrix *cm, gchar UL)
 gint
 ncm_matrix_cholesky_inverse (NcmMatrix *cm, gchar UL)
 {
-  gint ret = ncm_lapack_dpotri (UL, ncm_matrix_nrows (cm), ncm_matrix_data (cm), ncm_matrix_nrows (cm));
+  gint ret = ncm_lapack_dpotri (UL, ncm_matrix_nrows (cm), ncm_matrix_data (cm), ncm_matrix_tda (cm));
   return ret;
 }
 
@@ -1095,7 +1111,7 @@ ncm_matrix_cholesky_lndet (NcmMatrix *cm)
 
   for (i = 0; i < n; i++)
   {
-    const gdouble Lii = ncm_matrix_get (cm, i, i);
+    const gdouble Lii = fabs (ncm_matrix_get (cm, i, i));
     const gdouble ndetL = detL * Lii;
     if (G_UNLIKELY ((ndetL < lb) || (ndetL > ub)))
     {
@@ -1106,10 +1122,345 @@ ncm_matrix_cholesky_lndet (NcmMatrix *cm)
     else
       detL = ndetL;
   }
-
   return 2.0 * (log (detL) + exponent * M_LN2);
 }
 
+/**
+ * ncm_matrix_cholesky_solve:
+ * @cm: a #NcmMatrix
+ * @b: a #NcmVector
+ * @UL: char indicating 'U'pper or 'L'ower matrix
+ *
+ * Calculates inplace the Cholesky decomposition for a symmetric positive
+ * definite matrix and solve the system $A x = B$ where $A=$@cm and $B$=@b.
+ * 
+ */
+gint 
+ncm_matrix_cholesky_solve (NcmMatrix *cm, NcmVector *b, gchar UL)
+{
+	g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+	g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_vector_len (b));
+	g_assert_cmpuint (ncm_vector_stride (b), ==, 1);
+
+	return ncm_lapack_dposv (UL, ncm_matrix_nrows (cm), 1, 
+	                         ncm_matrix_data (cm), ncm_matrix_tda (cm), 
+	                         ncm_vector_data (b),  ncm_vector_len (b));
+}
+
+/**
+ * ncm_matrix_cholesky_solve2:
+ * @cm: a #NcmMatrix
+ * @b: a #NcmVector
+ * @UL: char indicating 'U'pper or 'L'ower matrix
+ *
+ * Using a previously computed Cholesky decomposition in @cm, through 
+ * ncm_matrix_cholesky_decomp(), solves the system $A x = B$ where 
+ * $A=$@cm and $B$=@b.
+ * 
+ */
+gint 
+ncm_matrix_cholesky_solve2 (NcmMatrix *cm, NcmVector *b, gchar UL)
+{
+	g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+	g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_vector_len (b));
+	g_assert_cmpuint (ncm_vector_stride (b), ==, 1);
+
+	return ncm_lapack_dpotrs (UL, ncm_matrix_nrows (cm), 1, 
+	                          ncm_matrix_data (cm), ncm_matrix_tda (cm), 
+	                          ncm_vector_data (b),  ncm_vector_len (b));
+}
+
+/**
+ * ncm_matrix_nearPD:
+ * @cm: a #NcmMatrix
+ * @UL: char indicating 'U'pper or 'L'ower matrix 
+ * @cholesky_decomp: if true substitue @cm for its Cholesky decomposition
+ * @maxiter: maximum number of iterations
+ * 
+ * Assuming that @cm is a symmetric matrix with data on @UL 
+ * side, computes the nearest positive definite matrix
+ * in the Frobenius norm. See [Higham (2002)][XHigham2002].
+ * The iterations stop when the Cholesky decomposition is valid.
+ * 
+ * Returns: the return value of the last Cholesky decomposition
+ */
+gint
+ncm_matrix_nearPD (NcmMatrix *cm, gchar UL, gboolean cholesky_decomp, const guint maxiter)
+{
+  const guint n   = ncm_matrix_ncols (cm);
+  NcmVector *eva  = ncm_vector_new (n);
+  NcmVector *diag = ncm_vector_new (n);
+  NcmMatrix *eve  = ncm_matrix_new (n, n);
+  NcmMatrix *D_S  = ncm_matrix_new (n, n);
+  NcmMatrix *R    = ncm_matrix_new (n, n);
+  GArray *isuppz  = g_array_new (FALSE, FALSE, sizeof (gint));
+  NcmLapackWS *ws = ncm_lapack_ws_new ();
+  gint neva       = 0;
+  gint ret, i, iter;
+
+  g_array_set_size (isuppz, 2 * n);
+
+  g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+
+  ncm_matrix_set_zero (D_S);
+  ncm_matrix_get_diag (cm, diag);
+
+  iter = 0;
+  while (TRUE)
+  {
+    gdouble min_pos_ev = GSL_POSINF;
+    ncm_matrix_sub (cm, D_S);
+
+    ncm_matrix_memcpy (R, cm);
+
+    ret = ncm_lapack_dsyevr ('V', 'A', UL, n, ncm_matrix_data (cm), ncm_matrix_tda (cm), 
+                             0.0, 0.0, 
+                             0, 0, 0.0, 
+                             &neva, ncm_vector_data (eva), 
+                             ncm_matrix_data (eve), ncm_matrix_tda (eve), 
+                             &g_array_index (isuppz, gint, 0), 
+                             ws);
+    g_assert_cmpint (ret, ==, 0);
+
+    if (neva == 0)
+      g_error ("ncm_matrix_nearPD: matrix is negative semi-definite.");
+
+    for (i = 0; i < neva; i++)
+    {
+      if (ncm_vector_get (eva, i) > 0.0)
+        min_pos_ev = MIN (ncm_vector_get (eva, i), min_pos_ev);
+    }
+    
+    for (i = 0; i < neva; i++)
+    {
+      if (ncm_vector_get (eva, i) < 0.0)
+        ncm_vector_set (eva, i, min_pos_ev * GSL_DBL_EPSILON);
+      cblas_dscal (n, sqrt (ncm_vector_get (eva, i)), ncm_matrix_ptr (eve, i, 0), 1);
+      //ncm_matrix_mul_row (eve, i, sqrt (ncm_vector_get (eva, i)));
+    }
+
+    /*ncm_vector_log_vals (eva, "EVA: ", "% 22.15g", TRUE);*/
+    /*ncm_matrix_log_vals (eve, "EVE: ", "% 22.15g");*/
+
+    cblas_dsyrk (CblasRowMajor, (UL == 'U') ? CblasUpper : CblasLower, 
+                 CblasTrans, n, neva,
+                 1.0, ncm_matrix_data (eve), ncm_matrix_tda (eve),
+                 0.0, ncm_matrix_data (cm), ncm_matrix_tda (cm));
+
+    ncm_matrix_memcpy (D_S, cm);
+    ncm_matrix_sub (D_S, R);
+
+    ncm_matrix_set_diag (cm, diag);
+
+    ncm_matrix_memcpy (R, cm);
+    if ((ret = ncm_matrix_cholesky_decomp (R, UL)) == 0)
+      break;
+    if (iter > maxiter)
+      break;
+
+    iter++;
+    /*printf ("CHOLESKY: %4d, ITER %6d\n", ncm_matrix_cholesky_decomp (R, UL), iter++);*/
+    /*ncm_matrix_log_vals (cm, "NewCM: ", "% 22.15g");*/
+  }
+
+  if (cholesky_decomp)
+    ncm_matrix_memcpy (cm, R);
+  
+  g_array_unref (isuppz);
+  ncm_lapack_ws_free (ws);
+  ncm_vector_free (eva);
+  ncm_vector_free (diag);
+  ncm_matrix_free (eve);
+  ncm_matrix_free (R);
+  ncm_matrix_free (D_S);
+
+  return ret;
+}
+
+/**
+ * ncm_matrix_sym_exp_cholesky:
+ * @cm: $M$ a #NcmMatrix
+ * @UL: char indicating 'U'pper or 'L'ower matrix 
+ * @exp_cm_dec: on exit this matrix contain the upper triangular matrix $U$ where $\exp(M) = U^\textsc{T}U$
+ * 
+ * Assuming that @cm is a symmetric matrix with data on @UL 
+ * side, computes the matrix exponential of @cm and its cholesky 
+ * decomposition.
+ */
+void 
+ncm_matrix_sym_exp_cholesky (NcmMatrix *cm, gchar UL, NcmMatrix *exp_cm_dec)
+{
+  const guint n   = ncm_matrix_ncols (cm);
+  NcmVector *eva  = ncm_vector_new (n);
+  NcmMatrix *eve  = exp_cm_dec;
+  GArray *tau     = g_array_new (FALSE, FALSE, sizeof (gdouble));
+  GArray *isuppz  = g_array_new (FALSE, FALSE, sizeof (gint));
+  NcmLapackWS *ws = ncm_lapack_ws_new ();
+  gint neva       = 0;
+  gint ret, i;
+
+  g_array_set_size (isuppz, 2 * n);
+  g_array_set_size (tau, n);
+
+  g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (exp_cm_dec), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (exp_cm_dec), ==, ncm_matrix_nrows (exp_cm_dec));
+
+  ret = ncm_lapack_dsyevr ('V', 'A', UL, n, ncm_matrix_data (cm), ncm_matrix_tda (cm), 
+                           0.0, 0.0, 
+                           0, 0, 0.0, 
+                           &neva, ncm_vector_data (eva), 
+                           ncm_matrix_data (eve), ncm_matrix_tda (eve), 
+                           &g_array_index (isuppz, gint, 0), 
+                           ws);
+  g_assert_cmpint (ret, ==, 0);
+
+  for (i = 0; i < n; i++)
+    cblas_dscal (n, exp (0.5 * ncm_vector_get (eva, i)), ncm_matrix_ptr (eve, i, 0), 1);
+
+  ret = ncm_lapack_dgeqrf (n, n, ncm_matrix_data (eve), ncm_matrix_tda (eve), &g_array_index (tau, gdouble, 0), ws);
+  g_assert_cmpint (ret, ==, 0);  
+
+  g_array_unref (isuppz);
+  g_array_unref (tau);
+  ncm_lapack_ws_free (ws);
+  ncm_vector_free (eva);
+}
+
+/**
+ * ncm_matrix_sym_posdef_log:
+ * @cm: $M$ a #NcmMatrix
+ * @UL: char indicating 'U'pper or 'L'ower matrix 
+ * @ln_cm: on exit this matrix contain the upper triangular matrix $U$ where $\exp(M) = U^\textsc{T}U$
+ * 
+ * Assuming that @cm is a symmetric matrix with data on @UL 
+ * side, computes the matrix logarithm of @cm.
+ */
+void 
+ncm_matrix_sym_posdef_log (NcmMatrix *cm, gchar UL, NcmMatrix *ln_cm)
+{
+  const guint n   = ncm_matrix_ncols (cm);
+  NcmVector *eva  = ncm_vector_new (n);
+  NcmMatrix *eve  = ncm_matrix_new (n, n);
+  NcmMatrix *temp = ncm_matrix_new (n, n);
+  GArray *isuppz  = g_array_new (FALSE, FALSE, sizeof (gint));
+  NcmLapackWS *ws = ncm_lapack_ws_new ();
+  gint neva       = 0;
+  gint ret, i;
+
+  g_array_set_size (isuppz, 2 * n);
+
+  g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (ln_cm), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (ln_cm), ==, ncm_matrix_nrows (ln_cm));
+
+  ret = ncm_lapack_dsyevr ('V', 'A', UL, n, ncm_matrix_data (cm), ncm_matrix_tda (cm), 
+                           0.0, 0.0, 
+                           0, 0, 0.0, 
+                           &neva, ncm_vector_data (eva), 
+                           ncm_matrix_data (eve), ncm_matrix_tda (eve), 
+                           &g_array_index (isuppz, gint, 0), 
+                           ws);
+  g_assert_cmpint (ret, ==, 0);
+
+  ncm_matrix_memcpy (temp, eve);
+  
+  for (i = 0; i < n; i++)
+  {
+    const gdouble e_val = ncm_vector_get (eva, i);
+    if (e_val <= 0.0)
+      g_error ("ncm_matrix_sym_posdef_log: cannot compute the logarithm, matrix not positive definite [%d, % 22.15g].", i, e_val);
+    cblas_dscal (n, log (e_val), ncm_matrix_ptr (eve, i, 0), 1);
+  }
+
+  cblas_dsyr2k (CblasRowMajor, (UL == 'U') ? CblasUpper : CblasLower, 
+                CblasTrans, n, n,
+                0.5, ncm_matrix_data (temp), ncm_matrix_tda (temp),
+                ncm_matrix_data (eve), ncm_matrix_tda (eve),
+                0.0, ncm_matrix_data (ln_cm), ncm_matrix_tda (ln_cm));
+  
+  g_array_unref (isuppz);
+  ncm_lapack_ws_free (ws);
+  ncm_vector_free (eva);
+  ncm_matrix_free (eve);
+  ncm_matrix_free (temp);
+}
+
+/**
+ * ncm_matrix_triang_to_sym:
+ * @cm: $M$ a #NcmMatrix
+ * @UL: char indicating 'U'pper or 'L'ower matrix
+ * @zero: whether it should first set to zero the other side of the matrix
+ * @sym: a #NcmMatrix to store the result
+ * 
+ * Assuming that @cm is a triangular square matrix with data on @UL 
+ * side, computes the symmetric matrix $M^\textsc{T}\times M$ if 
+ * @cm is upper triangular or $M\times M^\textsc{T}$ if it is
+ * lower triangular.
+ * 
+ * If @zero is TRUE it first sets to zero all elements above/below
+ * the diagonal for UL == 'L'/'U'. It should be TRUE whenever @cm
+ * has non-zero values at the other side. 
+ * 
+ */
+void 
+ncm_matrix_triang_to_sym (NcmMatrix *cm, gchar UL, gboolean zero, NcmMatrix *sym)
+{
+  const guint n = ncm_matrix_ncols (cm);
+  gint i;
+
+  g_assert_cmpuint (ncm_matrix_ncols (cm), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (sym), ==, ncm_matrix_nrows (cm));
+  g_assert_cmpuint (ncm_matrix_ncols (sym), ==, ncm_matrix_nrows (sym));
+
+  if (zero)
+  {
+    if (UL == 'U')
+    {
+      for (i = 0; i < n; i++)
+      {
+        gint j;
+        for (j = i + 1; j < n; j++)
+        {
+          ncm_matrix_set (cm, j, i, 0.0);
+        }
+      }
+    }
+    else if (UL == 'L')
+    {
+      for (i = 0; i < n; i++)
+      {
+        gint j;
+        for (j = i + 1; j < n; j++)
+        {
+          ncm_matrix_set (cm, i, j, 0.0);
+        }
+      }
+    }
+    else
+      g_assert_not_reached ();
+  }
+
+  ncm_matrix_memcpy (sym, cm);
+  
+  if (UL == 'U')
+  {
+    cblas_dtrmm (CblasRowMajor, 
+                 CblasLeft, CblasUpper, CblasTrans, CblasNonUnit, n, n,
+                 1.0, ncm_matrix_data (cm), ncm_matrix_tda (cm),
+                 ncm_matrix_data (sym), ncm_matrix_tda (sym));
+  }
+  else if (UL == 'L')
+  {
+    cblas_dtrmm (CblasRowMajor,
+                 CblasRight, CblasLower, CblasTrans, CblasNonUnit, n, n,
+                 1.0, ncm_matrix_data (cm), ncm_matrix_tda (cm),
+                 ncm_matrix_data (sym), ncm_matrix_tda (sym));
+  }
+  else
+    g_assert_not_reached ();
+}
+
 /**
  * ncm_matrix_log_vals:
  * @cm: a #NcmMatrix
@@ -1136,22 +1487,19 @@ ncm_matrix_log_vals (NcmMatrix *cm, gchar *prefix, gchar *format)
 }
 
 /**
- * ncm_matrix_fill_rand_cov:
+ * ncm_matrix_fill_rand_cor:
  * @cm: a square #NcmMatrix
- * @sigma_min: mininum standard deviation
- * @sigma_max: maximum standard deviation
  * @cor_level: correlation level parameter
  * @rng: a #NcmRNG
  * 
- * Overwrite @cm with a random covariance matrix, the
+ * Overwrite @cm with a random correlation matrix, the
  * parameter @cor_level controls the correlation between
  * entries the lower @cor_level more correlated the entries
  * are.
  *
- * 
  */
 void 
-ncm_matrix_fill_rand_cov (NcmMatrix *cm, const gdouble sigma_min, const gdouble sigma_max, const gdouble cor_level, NcmRNG *rng)
+ncm_matrix_fill_rand_cor (NcmMatrix *cm, const gdouble cor_level, NcmRNG *rng)
 {
   const guint n   = ncm_matrix_nrows (cm);
   const guint nm1 = n - 1;
@@ -1189,19 +1537,90 @@ ncm_matrix_fill_rand_cov (NcmMatrix *cm, const gdouble sigma_min, const gdouble
       }
     }
 
-    for (k = 0; k < n; k++)
-    {
-      const gdouble sigma_k = ncm_rng_uniform_gen (rng, sigma_min, sigma_max);
-      ncm_matrix_mul_col (cm, k, sigma_k);
-      ncm_matrix_mul_row (cm, k, sigma_k);
-    }
-
     ncm_matrix_free (P);
   }
 
   ncm_rng_unlock (rng);
 }
 
+/**
+ * ncm_matrix_fill_rand_cov:
+ * @cm: a square #NcmMatrix
+ * @sigma_min: mininum standard deviation
+ * @sigma_max: maximum standard deviation
+ * @cor_level: correlation level parameter
+ * @rng: a #NcmRNG
+ * 
+ * Overwrite @cm with a random covariance matrix, the
+ * parameter @cor_level controls the correlation between
+ * entries the lower @cor_level more correlated the entries
+ * are.
+ *
+ */
+void 
+ncm_matrix_fill_rand_cov (NcmMatrix *cm, const gdouble sigma_min, const gdouble sigma_max, const gdouble cor_level, NcmRNG *rng)
+{
+	const guint n = ncm_matrix_nrows (cm);
+	gint k;
+
+	g_assert_cmpfloat (sigma_min, >, 0.0);
+	g_assert_cmpfloat (sigma_max, >, sigma_min);
+	
+	ncm_matrix_fill_rand_cor (cm, cor_level, rng);
+		
+  ncm_rng_lock (rng);
+
+	for (k = 0; k < n; k++)
+	{
+		const gdouble sigma_k = ncm_rng_uniform_gen (rng, sigma_min, sigma_max);
+		ncm_matrix_mul_col (cm, k, sigma_k);
+		ncm_matrix_mul_row (cm, k, sigma_k);
+	}
+	
+  ncm_rng_unlock (rng);
+}
+
+/**
+ * ncm_matrix_fill_rand_cov2:
+ * @cm: a square #NcmMatrix
+ * @mu: mean #NcmVector
+ * @reltol_min: mininum standard deviation
+ * @reltol_max: maximum standard deviation
+ * @cor_level: correlation level parameter
+ * @rng: a #NcmRNG
+ * 
+ * Overwrite @cm with a random covariance matrix, the
+ * parameter @cor_level controls the correlation between
+ * entries the lower @cor_level more correlated the entries
+ * are.
+ *
+ */
+void 
+ncm_matrix_fill_rand_cov2 (NcmMatrix *cm, NcmVector *mu, const gdouble reltol_min, const gdouble reltol_max, const gdouble cor_level, NcmRNG *rng)
+{
+	const guint n = ncm_matrix_nrows (cm);
+	gint k;
+
+	g_assert_cmpfloat (reltol_min, >, 0.0);
+	g_assert_cmpfloat (reltol_max, >, reltol_min);
+	g_assert_cmpuint (n, ==, ncm_vector_len (mu));
+	
+	ncm_matrix_fill_rand_cor (cm, cor_level, rng);
+		
+  ncm_rng_lock (rng);
+
+	for (k = 0; k < n; k++)
+	{
+		const gdouble mu_k    = ncm_vector_get (mu, k);
+		const gdouble r_k     = exp (ncm_rng_uniform_gen (rng, log (reltol_min), log (reltol_max)));
+		const gdouble sigma_k = mu_k != 0.0 ? fabs (mu_k) * r_k : r_k;
+		ncm_matrix_mul_col (cm, k, sigma_k);
+		ncm_matrix_mul_row (cm, k, sigma_k);
+	}
+	
+  ncm_rng_unlock (rng);
+}
+
 /**
  * ncm_matrix_new_gsl_const: (skip)
  * @m: matrix from GNU Scientific Library (GSL)
@@ -1235,14 +1654,7 @@ ncm_matrix_fill_rand_cov (NcmMatrix *cm, const gdouble sigma_min, const gdouble
  *
  * Returns: A constant pointer to the (@i,@j)-th element of the matrix @cm.
  */
-/**
- * ncm_matrix_ref:
- * @cm: a #NcmMatrix
- *
- * Increase the reference count of @cm by one.
- *
- * Returns: (transfer full): @cm
- */
+
 /**
  * ncm_matrix_set:
  * @cm: a #NcmMatrix
@@ -1387,6 +1799,16 @@ ncm_matrix_fill_rand_cov (NcmMatrix *cm, const gdouble sigma_min, const gdouble
  * The length of the vector must be the same as the length of the column.
  *
  */
+/**
+ * ncm_matrix_set_row:
+ * @cm: a #NcmMatrix
+ * @n: row index
+ * @cv: a constant #NcmVector
+ *
+ * This function copies the elements of the vector @cv into the @n-th row of the matrix @cm.
+ * The length of the vector must be the same as the length of the row.
+ *
+ */
 /**
  * ncm_matrix_get_array:
  * @cm: a #NcmMatrix
