@@ -40,16 +40,35 @@
 #include "math/integral.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
-#include <cvodes/cvodes.h>
-#if HAVE_SUNDIALS_MAJOR == 2
-#include <cvodes/cvodes_dense.h>
-#include <cvodes/cvodes_band.h>
-#elif HAVE_SUNDIALS_MAJOR == 3
-#endif
+#include <cvode/cvode.h>
 #include <nvector/nvector_serial.h> 
+#include <sunnonlinsol/sunnonlinsol_fixedpoint.h> 
 #include <gsl/gsl_linalg.h>
 #endif /* NUMCOSMO_GIR_SCAN */
 
+struct _NcmOdeSplinePrivate
+{
+  gpointer cvode;
+  SUNNonlinearSolver NLS;
+  N_Vector y;
+  GArray *y_array;
+  GArray *x_array;
+  gdouble xi;
+  gdouble xf;
+  gdouble yi;
+	gdouble yf;
+  gdouble reltol;
+  gdouble abstol;
+  NcmOdeSplineDydx dydx;
+  gboolean s_init;
+  gboolean cvode_init;
+  gboolean hnil;
+  gboolean stop_hnil;
+  gboolean auto_abstol;
+  gdouble ini_step;
+  NcmModelCtrl *ctrl;
+};
+
 enum
 {
   PROP_0,
@@ -62,36 +81,53 @@ enum
   PROP_DYDX,
   PROP_SPLINE,
   PROP_STOP_HNIL,
+  PROP_AUTO_ABSTOL,
+  PROP_INI_STEP,
 };
 
-G_DEFINE_TYPE (NcmOdeSpline, ncm_ode_spline, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (NcmOdeSpline, ncm_ode_spline, G_TYPE_OBJECT);
+
+typedef struct _NcmOdeSplineDydxData
+{
+	NcmOdeSpline *os;
+  gpointer userdata;
+} NcmOdeSplineDydxData;
 
 static void
 ncm_ode_spline_init (NcmOdeSpline *os)
 {
-  os->cvode      = CVodeCreate (CV_ADAMS, CV_FUNCTIONAL);
-  os->cvode_init = FALSE;
-  os->y          = N_VNew_Serial (1);
-  os->y_array    = g_array_sized_new (FALSE, FALSE, sizeof (gdouble), 1000);
-  os->x_array    = g_array_sized_new (FALSE, FALSE, sizeof (gdouble), 1000);
-  os->xi         = GSL_NAN;
-  os->xf         = GSL_NAN;
-  os->yi         = GSL_NAN;
-	os->yf         = GSL_NAN;
-  os->reltol     = 0.0;
-  os->abstol     = 0.0;
-  os->dydx       = NULL;
-  os->s          = NULL;
-  os->s_init     = FALSE;
-  os->hnil       = FALSE;
-  os->stop_hnil  = FALSE;
-  os->ctrl       = ncm_model_ctrl_new (NULL);  
+  NcmOdeSplinePrivate * const self = os->priv = G_TYPE_INSTANCE_GET_PRIVATE (os, NCM_TYPE_ODE_SPLINE, NcmOdeSplinePrivate);
+
+  os->spline        = NULL;
+  self->cvode       = CVodeCreate (CV_ADAMS);
+  self->cvode_init  = FALSE;
+  self->y           = N_VNew_Serial (1);
+  self->NLS         = NULL;
+  self->y_array     = g_array_sized_new (FALSE, FALSE, sizeof (gdouble), 1000);
+  self->x_array     = g_array_sized_new (FALSE, FALSE, sizeof (gdouble), 1000);
+  self->xi          = GSL_NAN;
+  self->xf          = GSL_NAN;
+  self->yi          = GSL_NAN;
+	self->yf          = GSL_NAN;
+  self->reltol      = 0.0;
+  self->abstol      = 0.0;
+  self->dydx        = NULL;
+  self->s_init      = FALSE;
+  self->hnil        = FALSE;
+  self->stop_hnil   = FALSE;
+  self->auto_abstol = FALSE;
+  self->ini_step    = 0.0;
+  self->ctrl        = ncm_model_ctrl_new (NULL);
+
+  self->NLS = SUNNonlinSol_FixedPoint (self->y, 0);
+  NCM_CVODE_CHECK (self->NLS, "SUNNonlinSol_FixedPoint", 0, );
 }
 
 static void
 _ncm_ode_spline_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmOdeSpline *os = NCM_ODE_SPLINE (object);
+  NcmOdeSplinePrivate * const self = os->priv;
   g_return_if_fail (NCM_IS_ODE_SPLINE (object));
 
   switch (prop_id)
@@ -115,13 +151,20 @@ _ncm_ode_spline_set_property (GObject *object, guint prop_id, const GValue *valu
       ncm_ode_spline_set_yf (os, g_value_get_double (value));
       break;
     case PROP_DYDX:
-      os->dydx = (NcmOdeSplineDydx) g_value_get_pointer (value);
+      self->dydx = (NcmOdeSplineDydx) g_value_get_pointer (value);
       break;
     case PROP_SPLINE:
-      os->s = g_value_dup_object (value);
+      ncm_spline_clear (&os->spline);
+      os->spline = g_value_dup_object (value);
       break;
     case PROP_STOP_HNIL:
-      os->stop_hnil = g_value_get_boolean (value);
+      self->stop_hnil = g_value_get_boolean (value);
+      break;
+    case PROP_AUTO_ABSTOL:
+      ncm_ode_spline_auto_abstol (os, g_value_get_boolean (value));
+      break;
+    case PROP_INI_STEP:
+      ncm_ode_spline_set_ini_step (os, g_value_get_double (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -133,36 +176,43 @@ static void
 _ncm_ode_spline_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcmOdeSpline *os = NCM_ODE_SPLINE (object);
+  NcmOdeSplinePrivate * const self = os->priv;
   g_return_if_fail (NCM_IS_ODE_SPLINE (object));
 
   switch (prop_id)
   {
     case PROP_RELTOL:
-      g_value_set_double (value, os->reltol);
+      g_value_set_double (value, self->reltol);
       break;
     case PROP_ABSTOL:
-      g_value_set_double (value, os->abstol);
+      g_value_set_double (value, self->abstol);
       break;
     case PROP_XI:
-      g_value_set_double (value, os->xi);
+      g_value_set_double (value, self->xi);
       break;
     case PROP_XF:
-      g_value_set_double (value, os->xf);
+      g_value_set_double (value, self->xf);
       break;
     case PROP_YI:
-      g_value_set_double (value, os->yi);
+      g_value_set_double (value, self->yi);
       break;
     case PROP_YF:
-      g_value_set_double (value, os->yf);
+      g_value_set_double (value, self->yf);
       break;
     case PROP_DYDX:
-      g_value_set_pointer (value, os->dydx);
+      g_value_set_pointer (value, self->dydx);
       break;
     case PROP_SPLINE:
-      g_value_set_object (value, os->s);
+      g_value_set_object (value, os->spline);
       break;
     case PROP_STOP_HNIL:
-      g_value_set_boolean (value, os->stop_hnil);
+      g_value_set_boolean (value, self->stop_hnil);
+      break;
+    case PROP_AUTO_ABSTOL:
+      g_value_set_boolean (value, self->auto_abstol);
+      break;
+    case PROP_INI_STEP:
+      g_value_set_boolean (value, ncm_ode_spline_get_ini_step (os));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -174,11 +224,12 @@ static void
 _ncm_ode_spline_dispose (GObject *object)
 {
   NcmOdeSpline *os = NCM_ODE_SPLINE (object);
+  NcmOdeSplinePrivate * const self = os->priv;
 
-  ncm_spline_clear (&os->s);
-  g_clear_pointer (&os->x_array, g_array_unref);
-  g_clear_pointer (&os->y_array, g_array_unref);  
-  ncm_model_ctrl_clear (&os->ctrl);
+  ncm_spline_clear (&os->spline);
+  g_clear_pointer (&self->x_array, g_array_unref);
+  g_clear_pointer (&self->y_array, g_array_unref);  
+  ncm_model_ctrl_clear (&self->ctrl);
 
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_ode_spline_parent_class)->dispose (object);
@@ -188,16 +239,22 @@ static void
 _ncm_ode_spline_finalize (GObject *object)
 {
   NcmOdeSpline *os = NCM_ODE_SPLINE (object);
+  NcmOdeSplinePrivate * const self = os->priv;
 
-  if (os->cvode != NULL)
+  if (self->cvode != NULL)
   {
-    CVodeFree (&os->cvode);
-    os->cvode = NULL;
+    CVodeFree (&self->cvode);
+    self->cvode = NULL;
   }
-  if (os->y != NULL)
+  if (self->y != NULL)
   {
-    N_VDestroy (os->y);
-    os->y = NULL;
+    N_VDestroy (self->y);
+    self->y = NULL;
+  }
+  if (self->NLS != NULL)
+  {
+    SUNNonlinSolFree (self->NLS);
+    self->NLS = NULL;
   }
 
   /* Chain up : end */
@@ -282,27 +339,29 @@ ncm_ode_spline_class_init (NcmOdeSplineClass *klass)
                                                          "Whether treat hnil as error",
                                                          TRUE,
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_AUTO_ABSTOL,
+                                   g_param_spec_boolean ("auto-abstol",
+                                                         NULL,
+                                                         "Automatic abstol",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_INI_STEP,
+                                   g_param_spec_double ("ini-step",
+                                                        NULL,
+                                                        "Integration initial step size",
+                                                        0.0, G_MAXDOUBLE, 0.0,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
-typedef struct _NcmOdeSplineDydxData NcmOdeSplineDydxData;
-
-/**
- * NcFunctionParams:
- *
- * FIXME
- */
-struct _NcmOdeSplineDydxData
-{
-	NcmOdeSpline *os;
-  gpointer userdata;
-};
-
 static gint
 _ncm_ode_spline_f (realtype x, N_Vector y, N_Vector ydot, gpointer f_data)
 {
   NcmOdeSplineDydxData *dydx_data = (NcmOdeSplineDydxData *) f_data;
-  NV_Ith_S (ydot, 0) = dydx_data->os->dydx (NV_Ith_S (y, 0), x, dydx_data->userdata);
-  /*printf ("% 20.15g % 20.15g % 20.15g\n", x, NV_Ith_S (y, 0), NV_Ith_S (ydot, 0));*/
+  NcmOdeSplinePrivate * const self = dydx_data->os->priv;
+  NV_Ith_S (ydot, 0) = self->dydx (NV_Ith_S (y, 0), x, dydx_data->userdata);
+
   return 0;
 }
 
@@ -354,8 +413,9 @@ static gint
 _ncm_ode_spline_yf_root (realtype lambda, N_Vector y, realtype *gout, gpointer user_data)
 {
   NcmOdeSplineDydxData *dydx_data = (NcmOdeSplineDydxData *) user_data;
+  NcmOdeSplinePrivate * const self = dydx_data->os->priv;
 
-	gout[0] = (NV_Ith_S (y, 0) - dydx_data->os->yf);
+	gout[0] = (NV_Ith_S (y, 0) - self->yf);
 
 	return 0;
 } 
@@ -370,68 +430,80 @@ _ncm_ode_spline_yf_root (realtype lambda, N_Vector y, realtype *gout, gpointer u
 void
 ncm_ode_spline_prepare (NcmOdeSpline *os, gpointer userdata)
 {
+  NcmOdeSplinePrivate * const self = os->priv;
   NcmOdeSplineDydxData f_data = {os, userdata};
   gdouble x, x0;
   gint flag;
 
-  NV_Ith_S (os->y, 0) = os->yi;
-  
-  if (!os->cvode_init)
+  NV_Ith_S (self->y, 0) = self->yi;
+
+  if (self->auto_abstol)
   {
-    flag = CVodeInit (os->cvode, &_ncm_ode_spline_f, os->xi, os->y);
+    self->abstol = fabs (self->dydx (NV_Ith_S (self->y, 0), self->xi, userdata) * self->reltol * NCM_ODE_SPLINE_MIN_STEP);
+  }
+  else if ((self->yi == 0.0) && (self->abstol == 0.0))
+    g_error ("ncm_ode_spline_prepare: cannot integrate system where y_ini == 0.0 and abstol == 0.0.");
+  
+  if (!self->cvode_init)
+  {
+    flag = CVodeInit (self->cvode, &_ncm_ode_spline_f, self->xi, self->y);
     NCM_CVODE_CHECK (&flag, "CVodeInit", 1, );
+
+    flag = CVodeSetNonlinearSolver (self->cvode, self->NLS);
+    NCM_CVODE_CHECK (&flag, "CVodeSetNonlinearSolver", 1, );
     
-    os->cvode_init = TRUE;
+    self->cvode_init = TRUE;
   }
   else
   {
-    flag = CVodeReInit (os->cvode, os->xi, os->y);
+    flag = CVodeReInit (self->cvode, self->xi, self->y);
     NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, );
+        
+    flag = CVodeSetNonlinearSolver (self->cvode, self->NLS);
+    NCM_CVODE_CHECK (&flag, "CVodeSetNonlinearSolver", 1, );
   }
 
-  g_array_set_size (os->x_array, 0);
-  g_array_set_size (os->y_array, 0);
+  g_array_set_size (self->x_array, 0);
+  g_array_set_size (self->y_array, 0);
 
-  g_array_append_val (os->x_array, os->xi);
-  g_array_append_val (os->y_array, NV_Ith_S (os->y, 0));
+  g_array_append_val (self->x_array, self->xi);
+  g_array_append_val (self->y_array, NV_Ith_S (self->y, 0));
 
-  flag = CVodeSStolerances (os->cvode, os->reltol, os->abstol);
+  flag = CVodeSStolerances (self->cvode, self->reltol, self->abstol);
   NCM_CVODE_CHECK (&flag, "CVodeSStolerances", 1, );
   
-  flag = CVodeSetMaxNumSteps (os->cvode, NCM_INTEGRAL_PARTITION);
+  flag = CVodeSetMaxNumSteps (self->cvode, NCM_INTEGRAL_PARTITION);
   NCM_CVODE_CHECK (&flag, "CVodeSetMaxNumSteps", 1, );
   
-  flag = CVodeSetUserData (os->cvode, &f_data);
+  flag = CVodeSetUserData (self->cvode, &f_data);
   NCM_CVODE_CHECK (&flag, "CVodeSetUserData", 1, );
 
-#if defined (HAVE_SUNDIALS_2_5_0) || defined (HAVE_SUNDIALS_2_6_0)
-  if (os->xi != 0.0)
+  if (self->ini_step > 0.0)
   {
-    flag = CVodeSetInitStep (os->cvode, fabs (os->xi * 1.0e-11) * GSL_SIGN (os->xf - os->xi));
-    NCM_CVODE_CHECK (&flag, "CVodeSetInitStep", 1, );
+    flag = CVodeSetInitStep (self->cvode, self->ini_step);
+    NCM_CVODE_CHECK (&flag, "CVodeSetUserData", 1, );  
   }
-#endif
-  
-  x0 = os->xi;
-  if (!gsl_finite (os->dydx (NV_Ith_S (os->y, 0), x0, f_data.userdata)))
+
+  x0 = self->xi;
+  if (!gsl_finite (self->dydx (NV_Ith_S (self->y, 0), x0, f_data.userdata)))
     g_error ("ncm_ode_spline_prepare: not finite integrand.");
 
-	if (!gsl_finite (os->yf))
+	if (!gsl_finite (self->yf))
 	{
-		g_assert (gsl_finite (os->xf));
+		g_assert (gsl_finite (self->xf));
 		
-		flag = CVodeSetStopTime (os->cvode, os->xf);
+		flag = CVodeSetStopTime (self->cvode, self->xf);
 		NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, );
 
-		os->hnil = FALSE;
+		self->hnil = FALSE;
 		while (TRUE)
 		{
-			flag = CVode (os->cvode, os->xf, os->y, &x, CV_ONE_STEP);
+			flag = CVode (self->cvode, self->xf, self->y, &x, CV_ONE_STEP);
 			NCM_CVODE_CHECK (&flag, "ncm_ode_spline_prepare[CVode]", 1, );
 
-			if (G_UNLIKELY (os->hnil))
+			if (G_UNLIKELY (self->hnil))
 			{
-				if (os->stop_hnil)
+				if (self->stop_hnil)
 				{
 					g_error ("ncm_ode_spline_prepare: cannot integrate function %d.", flag);
 				}
@@ -439,36 +511,34 @@ ncm_ode_spline_prepare (NcmOdeSpline *os, gpointer userdata)
 					break;
 			}
 
-			/*if (((x0 != 0.0) ? fabs ((x - x0) / x0) : fabs (x - x0)) > NCM_ODE_SPLINE_MIN_STEP)*/
 			if (x > x0 + fabs (x0) * NCM_ODE_SPLINE_MIN_STEP)
 			{
-				g_array_append_val (os->x_array, x);
-				g_array_append_val (os->y_array, NV_Ith_S (os->y, 0));
-				/*printf ("% 20.15g % 20.15g\n", x, NV_Ith_S (os->y, 0));*/
+				g_array_append_val (self->x_array, x);
+				g_array_append_val (self->y_array, NV_Ith_S (self->y, 0));
 				x0 = x;
 			}
 
-			if (x == os->xf)
+			if (x == self->xf)
 				break;
 		}
 	}
 	else
 	{
-		const gdouble xf = (os->xi != 0.0) ? os->xi * 2.0 : 1.0;
+		const gdouble xf = (self->xi != 0.0) ? self->xi * 2.0 : 1.0;
 		gdouble last_y = GSL_NEGINF;
 		
-		flag = CVodeRootInit (os->cvode, 1, &_ncm_ode_spline_yf_root);
+		flag = CVodeRootInit (self->cvode, 1, &_ncm_ode_spline_yf_root);
 		NCM_CVODE_CHECK (&flag, "CVodeRootInit", 1, );    
 		
-		os->hnil = FALSE;
+		self->hnil = FALSE;
 		while (TRUE)
 		{
-			flag = CVode (os->cvode, xf, os->y, &x, CV_ONE_STEP);
+			flag = CVode (self->cvode, xf, self->y, &x, CV_ONE_STEP);
 			NCM_CVODE_CHECK (&flag, "ncm_ode_spline_prepare[CVode]", 1, );
 
-			if (G_UNLIKELY (os->hnil))
+			if (G_UNLIKELY (self->hnil))
 			{
-				if (os->stop_hnil)
+				if (self->stop_hnil)
 				{
 					g_error ("ncm_ode_spline_prepare: cannot integrate function %d.", flag);
 				}
@@ -478,27 +548,27 @@ ncm_ode_spline_prepare (NcmOdeSpline *os, gpointer userdata)
 
 			if (x > x0 + fabs (x0) * NCM_ODE_SPLINE_MIN_STEP)
 			{
-				g_array_append_val (os->x_array, x);
-				g_array_append_val (os->y_array, NV_Ith_S (os->y, 0));
+				g_array_append_val (self->x_array, x);
+				g_array_append_val (self->y_array, NV_Ith_S (self->y, 0));
 				x0 = x;
 			}
 
 			/* Possible problem if the integrand stalls for a long time but resume growing afterwards! ATT */
-			if (NV_Ith_S (os->y, 0) == last_y)
+			if (NV_Ith_S (self->y, 0) == last_y)
 				break;
-			last_y = NV_Ith_S (os->y, 0);
+			last_y = NV_Ith_S (self->y, 0);
 			
 			if (flag == CV_ROOT_RETURN)
 				break;
 		}
 
-		if (ncm_cmp (last_y, os->yf, 1.0e-2, 0.0) != 0)
+		if (ncm_cmp (last_y, self->yf, 1.0e-2, 0.0) != 0)
 			g_warning ("ncm_ode_spline_prepare: system has saturated at `% 22.15g' before attaining the required final value `% 22.15g'.",
-			           last_y, os->yf);
+			           last_y, self->yf);
 	}
 	
-  ncm_spline_set_array (os->s, os->x_array, os->y_array, TRUE);
-  os->s_init = TRUE;
+  ncm_spline_set_array (os->spline, self->x_array, self->y_array, TRUE);
+  self->s_init = TRUE;
 }
 
 /**
@@ -556,7 +626,8 @@ ncm_ode_spline_set_interval (NcmOdeSpline *os, gdouble yi, gdouble xi, gdouble x
 void 
 ncm_ode_spline_set_reltol (NcmOdeSpline *os, gdouble reltol)
 {
-  os->reltol = reltol;
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->reltol = reltol;
 }
 
 /**
@@ -570,7 +641,8 @@ ncm_ode_spline_set_reltol (NcmOdeSpline *os, gdouble reltol)
 void 
 ncm_ode_spline_set_abstol (NcmOdeSpline *os, gdouble abstol)
 {
-  os->abstol = abstol;
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->abstol = abstol;
 }
 
 /**
@@ -584,7 +656,8 @@ ncm_ode_spline_set_abstol (NcmOdeSpline *os, gdouble abstol)
 void 
 ncm_ode_spline_set_xi (NcmOdeSpline *os, gdouble xi)
 {
-  os->xi = xi;
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->xi = xi;
 }
 
 /**
@@ -598,8 +671,9 @@ ncm_ode_spline_set_xi (NcmOdeSpline *os, gdouble xi)
 void 
 ncm_ode_spline_set_xf (NcmOdeSpline *os, gdouble xf)
 {
-  os->xf = xf;
-	if (gsl_finite (os->yf))
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->xf = xf;
+	if (gsl_finite (self->yf))
 	{
 		g_warning ("ncm_ode_spline_set_xf: setting xf when yf was also set, yf will take precedence.");
 	}
@@ -616,7 +690,8 @@ ncm_ode_spline_set_xf (NcmOdeSpline *os, gdouble xf)
 void 
 ncm_ode_spline_set_yi (NcmOdeSpline *os, gdouble yi)
 {
-  os->yi = yi;
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->yi = yi;
 }
 
 /**
@@ -630,9 +705,68 @@ ncm_ode_spline_set_yi (NcmOdeSpline *os, gdouble yi)
 void 
 ncm_ode_spline_set_yf (NcmOdeSpline *os, gdouble yf)
 {
-  os->yf = yf;
-	if (gsl_finite (os->xf))
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->yf = yf;
+	if (gsl_finite (self->xf))
 	{
 		g_warning ("ncm_ode_spline_set_yf: setting yf when xf was also set, yf will take precedence.");
 	}
 }
+
+/**
+ * ncm_ode_spline_auto_abstol:
+ * @os: a #NcmOdeSpline
+ * @on: Whether to turn on the auto-abstol
+ *
+ * If @on is TRUE, the object uses the value of $\mathrm{d}y_i$ to estimate the 
+ * abstol as $T_\mathrm{abs} = \dot{y}_i \mathrm{d}t_m T_\mathrm{rel}$,
+ * where $T_\mathrm{rel}$ is the relative tolerance and $\mathrm{d}t_m$ is the 
+ * minimum time step #NCM_ODE_SPLINE_MIN_STEP. Useful when computing integrals as ODEs.
+ */
+void 
+ncm_ode_spline_auto_abstol (NcmOdeSpline *os, gboolean on)
+{
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->auto_abstol = on;
+}
+
+/**
+ * ncm_ode_spline_set_ini_step:
+ * @os: a #NcmOdeSpline
+ * @ini_step: the initial step
+ * 
+ * Sets a guess for the initial step size. If @ini_step is
+ * zero it uses the automatic determination based on the
+ * tolerances.
+ * 
+ */
+void 
+ncm_ode_spline_set_ini_step (NcmOdeSpline *os, gdouble ini_step)
+{
+  NcmOdeSplinePrivate * const self = os->priv;
+  self->ini_step = ini_step;
+}
+
+/**
+ * ncm_ode_spline_get_ini_step:
+ * @os: a #NcmOdeSpline
+ * 
+ * Gets the current guess for the initial step size. 
+ * 
+ * Returns: the current value of the initial guess (zero means disabled).
+ */
+gdouble
+ncm_ode_spline_get_ini_step (NcmOdeSpline *os)
+{
+  NcmOdeSplinePrivate * const self = os->priv;
+  return self->ini_step;
+}
+
+/**
+ * ncm_ode_spline_peek_spline:
+ * @os: a #NcmOdeSpline
+ *
+ * Peeks at the last prepared spline.
+ * 
+ * Returns: (transfer none): the last prepared spline.
+ */
