@@ -152,7 +152,7 @@ ncm_rng_class_init (NcmRNGClass *klass)
                                    g_param_spec_ulong ("seed",
                                                         NULL,
                                                         "Algorithm seed",
-                                                        0, G_MAXUINT32, 0,
+                                                        0, G_MAXULONG, 0,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (object_class,
@@ -549,3 +549,21 @@ ncm_rng_pool_get (const gchar *name)
  * FIXME
  * 
  */
+/**
+ * ncm_rng_beta_gen:
+ * @rng: a #NcmRNG
+ * @a: FIXME
+ * @b: FIXME
+ * 
+ * FIXME
+ * 
+ */
+/**
+ * ncm_rng_gamma_gen:
+ * @rng: a #NcmRNG
+ * @a: FIXME
+ * @b: FIXME
+ * 
+ * FIXME
+ * 
+ */
