@@ -49,6 +49,42 @@
 
 G_DEFINE_TYPE (NcmSplineCubicNotaknot, ncm_spline_cubic_notaknot, NCM_TYPE_SPLINE_CUBIC);
 
+static void
+ncm_spline_cubic_notaknot_init (NcmSplineCubicNotaknot *s)
+{
+  NCM_UNUSED (s);
+}
+
+static void
+_ncm_spline_cubic_notaknot_finalize (GObject *object)
+{
+
+  /* Chain up : end */
+	G_OBJECT_CLASS (ncm_spline_cubic_notaknot_parent_class)->finalize (object);
+}
+
+static const gchar *_ncm_spline_cubic_notaknot_name (NcmSpline *s);
+static void _ncm_spline_cubic_notaknot_prepare_base (NcmSpline *s);
+static void _ncm_spline_cubic_notaknot_prepare (NcmSpline *s);
+static gsize _ncm_spline_cubic_notaknot_min_size (const NcmSpline *s);
+static NcmSpline *_ncm_spline_cubic_notaknot_copy_empty (const NcmSpline *s);
+
+static void
+ncm_spline_cubic_notaknot_class_init (NcmSplineCubicNotaknotClass *klass)
+{
+	GObjectClass* object_class = G_OBJECT_CLASS (klass);
+	NcmSplineClass *s_class = NCM_SPLINE_CLASS (klass);
+
+	object_class->finalize = &_ncm_spline_cubic_notaknot_finalize;
+
+	s_class->name         = &_ncm_spline_cubic_notaknot_name;
+	s_class->prepare      = &_ncm_spline_cubic_notaknot_prepare;
+	s_class->prepare_base = &_ncm_spline_cubic_notaknot_prepare_base;
+	s_class->min_size     = &_ncm_spline_cubic_notaknot_min_size;
+	s_class->copy_empty   = &_ncm_spline_cubic_notaknot_copy_empty;
+}
+
+
 /**
  * ncm_spline_cubic_notaknot_new:
  *
@@ -88,14 +124,14 @@ _ncm_spline_cubic_notaknot_copy_empty (const NcmSpline *s)
 }
 
 static gsize
-_ncm_spline_notaknot_min_size (const NcmSpline *s)
+_ncm_spline_cubic_notaknot_min_size (const NcmSpline *s)
 {
   NCM_UNUSED (s);
 	return 6;
 }
 
 static const gchar *
-_ncm_spline_notaknot_name (NcmSpline *s)
+_ncm_spline_cubic_notaknot_name (NcmSpline *s)
 {
   NCM_UNUSED (s);
   return "NcmSplineCubicNotaknot";
@@ -104,27 +140,27 @@ _ncm_spline_notaknot_name (NcmSpline *s)
 #define NUMCOSMO_CHECK_SPLINE_NODES 1
 
 static void
-_ncm_spline_notaknot_prepare_base (NcmSpline *s)
+_ncm_spline_cubic_notaknot_prepare_base (NcmSpline *s)
 {
 	NcmSplineCubic *sc = NCM_SPLINE_CUBIC (s);
-	const size_t size = s->len;
-	const size_t n = size - 1;
-	const size_t sys_size = size - 2;
-	const size_t nm1 = n - 1;
-	const size_t nm2 = nm1 - 1;
-	const size_t nm3 = nm2 - 1;
-	const gdouble h_0 = ncm_vector_get (s->xv, 1) - ncm_vector_get (s->xv, 0);
-	const gdouble h_1 = ncm_vector_get (s->xv, 2) - ncm_vector_get (s->xv, 1);
-	const gdouble h_nm1 = ncm_vector_get (s->xv, n)   - ncm_vector_get (s->xv, nm1);
-	const gdouble h_nm2 = ncm_vector_get (s->xv, nm1) - ncm_vector_get (s->xv, nm2);
+	const size_t size            = s->len;
+	const size_t n               = size - 1;
+	const size_t sys_size        = size - 2;
+	const size_t nm1             = n - 1;
+	const size_t nm2             = nm1 - 1;
+	const size_t nm3             = nm2 - 1;
+	const gdouble h_0            = ncm_vector_get (s->xv, 1) - ncm_vector_get (s->xv, 0);
+	const gdouble h_1            = ncm_vector_get (s->xv, 2) - ncm_vector_get (s->xv, 1);
+	const gdouble h_nm1          = ncm_vector_get (s->xv, n)   - ncm_vector_get (s->xv, nm1);
+	const gdouble h_nm2          = ncm_vector_get (s->xv, nm1) - ncm_vector_get (s->xv, nm2);
   const gdouble h_0_p_2h_1     = h_0   + 2.0 * h_1;
   const gdouble h_nm1_p_2h_nm2 = h_nm1 + 2.0 * h_nm2;
-  const gdouble ydiff_0 = ncm_vector_get (s->yv, 1) - ncm_vector_get (s->yv, 0);
-  const gdouble ydiff_1 = ncm_vector_get (s->yv, 2) - ncm_vector_get (s->yv, 1);
-  register gdouble h_i        = h_0;
-  register gdouble h_ip1      = h_1;
-  register gdouble ydiff_i   = ydiff_0;
-  register gdouble ydiff_ip1 = ydiff_1;
+  const gdouble ydiff_0        = ncm_vector_get (s->yv, 1) - ncm_vector_get (s->yv, 0);
+  const gdouble ydiff_1        = ncm_vector_get (s->yv, 2) - ncm_vector_get (s->yv, 1);
+  register gdouble h_i         = h_0;
+  register gdouble h_ip1       = h_1;
+  register gdouble ydiff_i     = ydiff_0;
+  register gdouble ydiff_ip1   = ydiff_1;
   
 	NcmVector *g = sc->g;
 	sc->g = sc->c;
@@ -135,7 +171,7 @@ _ncm_spline_notaknot_prepare_base (NcmSpline *s)
 #ifdef NUMCOSMO_CHECK_SPLINE_NODES
   if (h_0 < 0.0 || h_1 < 0.0 || h_nm1 < 0.0)
   {                                                                          
-    g_error ("_ncm_spline_notaknot_prepare_base: in node [0, 1, 2, %zu, %zu] (% 20.15g, % 20.15g, % 20.15g, % 20.15g, % 20.15g), (% 20.15g, % 20.15g, % 20.15g, % 20.15g, % 20.15g).", nm1, n,
+    g_error ("_ncm_spline_cubic_notaknot_prepare_base: in node [0, 1, 2, %zu, %zu] (% 20.15g, % 20.15g, % 20.15g, % 20.15g, % 20.15g), (% 20.15g, % 20.15g, % 20.15g, % 20.15g, % 20.15g).", nm1, n,
              ncm_vector_get (s->xv, 0), ncm_vector_get (s->xv, 1), ncm_vector_get (s->xv, 2), ncm_vector_get (s->xv, nm1), ncm_vector_get (s->xv, n),
              ncm_vector_get (s->yv, 0), ncm_vector_get (s->yv, 1), ncm_vector_get (s->yv, 2), ncm_vector_get (s->yv, nm1), ncm_vector_get (s->yv, n));
   }
@@ -151,7 +187,7 @@ _ncm_spline_notaknot_prepare_base (NcmSpline *s)
 #ifdef NUMCOSMO_CHECK_SPLINE_NODES
     if (h_ip1 <= 0.0)
     {
-      g_error ("_ncm_spline_notaknot_prepare_base: in node %zd of %zd x_ip2 = % 20.15g and x_ip1 = % 20.15g, y_ip2 = % 20.15g and y_ip1 = % 20.15g.", 
+      g_error ("_ncm_spline_cubic_notaknot_prepare_base: in node %zd of %zd x_ip2 = % 20.15g and x_ip1 = % 20.15g, y_ip2 = % 20.15g and y_ip1 = % 20.15g.", 
                i, n,
                ncm_vector_get (s->xv, i + 2), ncm_vector_get (s->xv, i + 1),
                ncm_vector_get (s->yv, i + 2), ncm_vector_get (s->yv, i + 1));
@@ -159,8 +195,8 @@ _ncm_spline_notaknot_prepare_base (NcmSpline *s)
 #endif
     
     {
-      const gdouble g_i       = 1.0 / h_i;
-      const gdouble g_ip1     = 1.0 / h_ip1;
+      const gdouble g_i   = 1.0 / h_i;
+      const gdouble g_ip1 = 1.0 / h_ip1;
 
       ncm_vector_fast_set (sc->offdiag, i, h_ip1);
       ncm_vector_fast_set (sc->diag,    i, 2.0 * (h_ip1 + h_i));
@@ -170,8 +206,8 @@ _ncm_spline_notaknot_prepare_base (NcmSpline *s)
 
   
 	{
-		const gdouble g_0     = 1.0 / h_0;
-		const gdouble g_1     = 1.0 / h_1;
+		const gdouble g_0 = 1.0 / h_0;
+		const gdouble g_1 = 1.0 / h_1;
     
     ncm_vector_fast_set (sc->c,         0, 3.0 * (ydiff_1 * g_1 -  ydiff_0 * g_0) / h_0_p_2h_1);
     ncm_vector_fast_set (sc->c,         1, ncm_vector_fast_get (sc->c, 0) * h_1 / (h_1 + h_0));
@@ -218,21 +254,21 @@ _ncm_spline_notaknot_prepare_base (NcmSpline *s)
 }
 
 static void
-_ncm_spline_notaknot_prepare (NcmSpline *s)
+_ncm_spline_cubic_notaknot_prepare (NcmSpline *s)
 {
 	NcmSplineCubic *sc = NCM_SPLINE_CUBIC (s);
-	const size_t size = s->len;
-	const size_t n = size - 1;
+	const size_t size  = s->len;
+	const size_t n     = size - 1;
 	size_t i;
 
-	_ncm_spline_notaknot_prepare_base (s);
+	_ncm_spline_cubic_notaknot_prepare_base (s);
 
 	for (i = 0; i < n; i++)
 	{
-		const gdouble dx = ncm_vector_get (s->xv, i + 1) - ncm_vector_get (s->xv, i);
-		const gdouble dy = ncm_vector_get (s->yv, i + 1) - ncm_vector_get (s->yv, i);
+		const gdouble dx    = ncm_vector_get (s->xv, i + 1) - ncm_vector_get (s->xv, i);
+		const gdouble dy    = ncm_vector_get (s->yv, i + 1) - ncm_vector_get (s->yv, i);
 		const gdouble c_ip1 = ncm_vector_fast_get (sc->c, i + 1);
-		const gdouble c_i = ncm_vector_fast_get (sc->c, i);
+		const gdouble c_i   = ncm_vector_fast_get (sc->c, i);
 
 		ncm_vector_fast_set (sc->b, i, (dy / dx) - dx * (c_ip1 + 2.0 * c_i) / 3.0);
 		ncm_vector_fast_set (sc->d, i, (c_ip1 - c_i) / (3.0 * dx));
@@ -240,32 +276,3 @@ _ncm_spline_notaknot_prepare (NcmSpline *s)
 
 	return;
 }
-
-static void
-ncm_spline_cubic_notaknot_init (NcmSplineCubicNotaknot *object)
-{
-  NCM_UNUSED (object);
-}
-
-static void
-ncm_spline_cubic_notaknot_finalize (GObject *object)
-{
-
-  /* Chain up : end */
-	G_OBJECT_CLASS (ncm_spline_cubic_notaknot_parent_class)->finalize (object);
-}
-
-static void
-ncm_spline_cubic_notaknot_class_init (NcmSplineCubicNotaknotClass *klass)
-{
-	GObjectClass* object_class = G_OBJECT_CLASS (klass);
-	NcmSplineClass *s_class = NCM_SPLINE_CLASS (klass);
-
-	s_class->name         = &_ncm_spline_notaknot_name;
-	s_class->prepare      = &_ncm_spline_notaknot_prepare;
-	s_class->prepare_base = &_ncm_spline_notaknot_prepare_base;
-	s_class->min_size     = &_ncm_spline_notaknot_min_size;
-	s_class->copy_empty   = &_ncm_spline_cubic_notaknot_copy_empty;
-
-	object_class->finalize = ncm_spline_cubic_notaknot_finalize;
-}
