@@ -29,7 +29,10 @@
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_spline.h>
+
+#ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_math.h>
+#endif /* NUMCOSMO_GIR_SCAN */
 
 G_BEGIN_DECLS
 
@@ -39,6 +42,7 @@ G_BEGIN_DECLS
  * @NCM_SPLINE_FUNCTION_2x2POINTS: FIXME
  * @NCM_SPLINE_FUNCTION_SPLINE: FIXME
  * @NCM_SPLINE_FUNCTION_SPLINE_LNKNOT: FIXME
+ * @NCM_SPLINE_FUNCTION_SPLINE_SINHKNOT: FIXME
  * 
  * FIXME
  */ 
@@ -48,11 +52,12 @@ typedef enum _NcmSplineFuncType
   NCM_SPLINE_FUNCTION_2x2POINTS,
   NCM_SPLINE_FUNCTION_SPLINE,
   NCM_SPLINE_FUNCTION_SPLINE_LNKNOT,
+  NCM_SPLINE_FUNCTION_SPLINE_SINHKNOT,
 } NcmSplineFuncType;
 
 void ncm_spline_set_func (NcmSpline *s, NcmSplineFuncType ftype, gsl_function *F, gdouble xi, gdouble xf, gsize max_nodes, gdouble rel_error);
 
-#define NCM_SPLINE_FUNC_DEFAULT_MAX_NODES 10000
+#define NCM_SPLINE_FUNC_DEFAULT_MAX_NODES 10000000
 #define NCM_SPLINE_KNOT_DIFF_TOL (GSL_DBL_EPSILON * 1.0e2)
 
 G_END_DECLS
