@@ -38,7 +38,7 @@
 #include "build_cfg.h"
 
 #include "math/ncm_util.h"
-#include "math/memory_pool.h"
+#include "math/ncm_memory_pool.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
 #include <gsl/gsl_sf_legendre.h>
@@ -51,12 +51,6 @@
 #include <gsl/gsl_sf_hyperg.h>
 
 #include <cvode/cvode.h>
-#if HAVE_SUNDIALS_MAJOR == 2
-#include <cvode/cvode_dense.h>
-#elif HAVE_SUNDIALS_MAJOR == 3
-#include <cvodes/cvodes_direct.h>
-#endif
-
 #ifdef NUMCOSMO_HAVE_FFTW3
 #include <fftw3.h>
 #endif /* NUMCOSMO_HAVE_FFTW3 */
@@ -718,6 +712,100 @@ ncm_d3exprel (const gdouble x)
   return 0.25 * gsl_sf_hyperg_1F1_int (4, 5, x);
 }
 
+/**
+ * ncm_util_sinh1:
+ * @x: a double
+ *
+ * Returns: $\frac{\sinh(x)}{x}$
+ */
+gdouble 
+ncm_util_sinh1 (const gdouble x)
+{
+	const gdouble cut = 9.0 / 10.0;
+	if (fabs (x) < cut)
+	{
+		const gdouble x2 = x * x;
+		gdouble d        = 1.0;
+		gdouble x2n      = 1.0;
+		gdouble p        = 2.0;
+		gdouble res      = 0.0;
+		gint n;
+		
+		for (n = 0; n < 9; n++)
+		{
+			res += x2n / d;
+			
+			d   *= (p * (p + 1.0));
+			x2n *= x2;
+			p   += 2.0;
+		}
+
+		return res;
+	}
+	else
+	{
+		return sinh (x) / x;
+	}
+}
+
+/**
+ * ncm_util_sinh3:
+ * @x: a double
+ *
+ * Returns: $\frac{\sinh(x)-x}{x^3/3!}$
+ */
+gdouble 
+ncm_util_sinh3 (const gdouble x)
+{
+	const gdouble cut = 9.0 / 10.0;
+	if (fabs (x) < cut)
+	{
+		const gdouble x2 = x * x;
+		gdouble d        = 1.0;
+		gdouble x2n      = 1.0;
+		gdouble p        = 4.0;
+		gdouble res      = 0.0;
+		gint n;
+		
+		for (n = 0; n < 8; n++)
+		{
+			res += x2n / d;
+			
+			d   *= (p * (p + 1.0));
+			x2n *= x2;
+			p   += 2.0;
+		}
+
+		return res;
+	}
+	else
+	{
+		return (sinh (x) - x) * 6.0 / gsl_pow_3 (x);
+	}
+}
+
+/**
+ * ncm_util_sinhx_m_xcoshx_x3:
+ * @x: a double
+ *
+ * Returns: $\frac{\sinh(x)-x\cosh(x)}{x^3}$
+ */
+gdouble 
+ncm_util_sinhx_m_xcoshx_x3 (const gdouble x)
+{
+	const gdouble cut = 9.0 / 10.0;
+	const gdouble shx = sinh (x);
+
+	if (fabs (x) < cut)
+	{
+		return ncm_util_sinh3 (x) / 6.0 - gsl_pow_2 (ncm_util_sinh1 (x)) / (sqrt (1.0 + shx * shx) + 1.0);
+	}
+	else
+	{
+		return (shx - x * sqrt (1.0 + shx * shx)) / gsl_pow_3 (x);
+	}
+}
+
 /**
  * ncm_cmp:
  * @x: a double
@@ -733,7 +821,7 @@ ncm_d3exprel (const gdouble x)
 gint
 ncm_cmp (gdouble x, gdouble y, const gdouble reltol, const gdouble abstol)
 {
-  if (G_UNLIKELY (x == 0.0 && y == 0.0))
+  if (G_UNLIKELY ((x == 0.0) && (y == 0.0)))
     return 0;
   else
   {
@@ -891,6 +979,33 @@ ncm_complex_clear (NcmComplex **c)
  * 
  */
 
+/**
+ * ncm_util_position_angle:
+ * @ra1: Right ascension of object 1 
+ * @dec1: Declination of object 1 
+ * @ra2: Right ascension of object 2 
+ * @dec2: Declination of object 2
+ *  
+ * Computes the on-sky position angle (East of North) between object1 (@ra1, @dec1) and object2 (@ra2, dec2).
+ * The input coordinates ((@ra1, @dec1), (@ra2, @dec2)) must be given in decimal degrees.  
+ * 
+ * Returns: the position angle in radians 
+ */
+
+/**
+ * ncm_util_great_circle_distance:
+ * @ra1: Right ascension of object 1 
+ * @dec1: Declination of object 1 
+ * @ra2: Right ascension of object 2 
+ * @dec2: Declination of object 2
+ *  
+ * Compute the great circle distance (or separation, as defined in astropy) between poistion 1 (@ra1, @dec1) and position 2 (@ra2, @dec2).  
+ * See [Great-circle distance](https://en.wikipedia.org/wiki/Great-circle_distance), in particular the Vincenty equation (implemented here).
+ * The input coordinates ((@ra1, @dec1), (@ra2, @dec2)) must be given in decimal degrees.  
+ * 
+ * Returns: the great circle distance in decimal degrees
+ */
+
 /**
  * ncm_util_cvode_check_flag:
  * @flagvalue: FIXME
@@ -969,11 +1084,11 @@ ncm_util_cvode_print_stats (gpointer cvode)
   flag = CVodeGetNumNonlinSolvConvFails(cvode, &nconvfail);
   ncm_util_cvode_check_flag (&flag, "CVodeGetNumNonlinSolvConvFails", 1);
 
-  flag = CVDlsGetNumJacEvals (cvode, &njaceval);
-  ncm_util_cvode_check_flag (&flag, "CVDlsGetNumJacEvals", 1);
+  flag = CVodeGetNumJacEvals (cvode, &njaceval);
+  ncm_util_cvode_check_flag (&flag, "CVodeGetNumJacEvals", 1);
 
-  flag = CVDlsGetNumRhsEvals (cvode, &ndiffjaceval);
-  ncm_util_cvode_check_flag (&flag, "CVDlsGetNumRhsEvals", 1);
+  flag = CVodeGetNumRhsEvals (cvode, &ndiffjaceval);
+  ncm_util_cvode_check_flag (&flag, "CVodeGetNumRhsEvals", 1);
 
   flag = CVodeGetNumGEvals(cvode, &nrooteval);
   ncm_util_cvode_check_flag (&flag, "CVodeGetNumGEvals", 1);
