@@ -44,16 +44,6 @@
 #include "math/ncm_util.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
-#ifdef HAVE_BLAS
-#  ifdef HAVE_MKL_CBLAS_H
-#    include <mkl_cblas.h>
-#  elif defined (HAVE_CBLAS_H)
-#    include <cblas.h>
-#  else
-#    include <gsl/gsl_cblas.h>
-#  endif
-#endif
-
 #include <complex.h>
 #ifdef NUMCOSMO_HAVE_FFTW3
 #include <fftw3.h>
@@ -68,6 +58,113 @@ enum
 
 G_DEFINE_TYPE (NcmVector, ncm_vector, G_TYPE_OBJECT);
 
+static void
+ncm_vector_init (NcmVector *v)
+{
+  v->pdata = NULL;
+  v->pfree = NULL;
+  v->type = 0;
+  memset (&v->vv, 0, sizeof (gsl_vector_view));
+}
+
+static void
+_ncm_vector_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+  NcmVector *v = NCM_VECTOR (object);
+  g_return_if_fail (NCM_IS_VECTOR (object));
+
+  switch (prop_id)
+  {
+    case PROP_VALS:
+    {
+      GVariant *var = ncm_vector_get_variant (v);
+      g_value_take_variant (value, var);
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+_ncm_vector_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  NcmVector *v = NCM_VECTOR (object);
+  g_return_if_fail (NCM_IS_VECTOR (object));
+
+  switch (prop_id)
+  {
+    case PROP_VALS:
+    {
+      GVariant *var = g_value_get_variant (value);
+      ncm_vector_set_from_variant (v, var);
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+_ncm_vector_dispose (GObject *object)
+{
+  NcmVector *cv = NCM_VECTOR (object);
+
+  if (cv->pdata != NULL)
+  {
+    g_assert (cv->pfree != NULL);
+    cv->pfree (cv->pdata);
+    cv->pdata = NULL;
+    cv->pfree = NULL;
+  }
+
+  /* Chain up : end */
+  G_OBJECT_CLASS (ncm_vector_parent_class)->dispose (object);
+}
+
+static void
+_ncm_vector_finalize (GObject *object)
+{
+  NcmVector *cv = NCM_VECTOR (object);
+  switch (cv->type)
+  {
+    case NCM_VECTOR_SLICE:
+      g_slice_free1 (sizeof (gdouble) * ncm_vector_len (cv) * ncm_vector_stride (cv), ncm_vector_data (cv));
+      break;
+    case NCM_VECTOR_ARRAY:
+    case NCM_VECTOR_MALLOC:
+    case NCM_VECTOR_GSL_VECTOR:
+    case NCM_VECTOR_DERIVED:
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+  cv->vv.vector.data = NULL;
+
+  /* Chain up : end */
+  G_OBJECT_CLASS (ncm_vector_parent_class)->finalize (object);
+}
+
+static void
+ncm_vector_class_init (NcmVectorClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->set_property = &_ncm_vector_set_property;
+  object_class->get_property = &_ncm_vector_get_property;
+  object_class->dispose      = &_ncm_vector_dispose;
+  object_class->finalize     = &_ncm_vector_finalize;
+
+  g_object_class_install_property (object_class, PROP_VALS,
+                                   g_param_spec_variant ("values", NULL, "values",
+                                                         G_VARIANT_TYPE_ARRAY, NULL,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+
+}
+
 /**
  * ncm_vector_new:
  * @n: defines the size of the vector
@@ -102,11 +199,17 @@ NcmVector *
 ncm_vector_new_full (gdouble *d, gsize size, gsize stride, gpointer pdata, GDestroyNotify pfree)
 {
   NcmVector *cv = g_object_new (NCM_TYPE_VECTOR, NULL);
+
+	g_assert (d != NULL);
+	g_assert_cmpuint (size,   >, 0);
+  g_assert_cmpuint (stride, >, 0);
+	
   if (stride != 1)
     cv->vv = gsl_vector_view_array_with_stride (d, stride, size);
   else
     cv->vv = gsl_vector_view_array (d, size);
-  cv->type = NCM_VECTOR_DERIVED;
+
+	cv->type = NCM_VECTOR_DERIVED;
   g_assert ((pdata == NULL) || (pdata != NULL && pfree != NULL));
   cv->pdata = pdata;
   cv->pfree = pfree;
@@ -126,9 +229,10 @@ ncm_vector_new_full (gdouble *d, gsize size, gsize stride, gpointer pdata, GDest
 NcmVector *
 ncm_vector_new_fftw (guint size)
 {
-  gdouble *d = fftw_alloc_real (size);
+  gdouble *d    = fftw_alloc_real (size);
   NcmVector *cv = ncm_vector_new_full (d, size, 1, d, (GDestroyNotify) fftw_free);
-  cv->type = NCM_VECTOR_MALLOC;
+  cv->type      = NCM_VECTOR_MALLOC;
+
   return cv;
 }
 
@@ -178,10 +282,13 @@ ncm_vector_new_gsl_static (gsl_vector *gv)
 NcmVector *
 ncm_vector_new_array (GArray *a)
 {
-  NcmVector *cv = ncm_vector_new_full (&g_array_index (a, gdouble, 0), a->len, 1,
-                                       g_array_ref (a), (GDestroyNotify) &g_array_unref);
-  cv->type = NCM_VECTOR_ARRAY;
-  return cv;
+	g_assert_cmpint (a->len, >, 0);
+	{
+		NcmVector *cv = ncm_vector_new_full (&g_array_index (a, gdouble, 0), a->len, 1,
+		                                     g_array_ref (a), (GDestroyNotify) &g_array_unref);
+		cv->type = NCM_VECTOR_ARRAY;
+		return cv;
+	}
 }
 
 /**
@@ -201,7 +308,7 @@ ncm_vector_new_data_slice (gdouble *d, gsize size, gsize stride)
 {
   NcmVector *cv = ncm_vector_new_full (d, size, stride,
                                        NULL, NULL);
-  cv->type  = NCM_VECTOR_SLICE;
+  cv->type = NCM_VECTOR_SLICE;
   return cv;
 }
 
@@ -361,6 +468,49 @@ ncm_vector_const_new_variant (GVariant *var)
   return v;
 }
 
+
+/**
+ * ncm_vector_free:
+ * @cv: a #NcmVector
+ *
+ * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
+ * all memory allocated by @cv is released.
+ *
+ */
+void
+ncm_vector_free (NcmVector *cv)
+{
+  g_object_unref (cv);
+}
+
+/**
+ * ncm_vector_const_free:
+ * @cv: a constant #NcmVector
+ *
+ * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
+ * all memory allocated by @cv is released.
+ *
+ */
+void
+ncm_vector_const_free (const NcmVector *cv)
+{
+  ncm_vector_free (NCM_VECTOR (cv));
+}
+
+/**
+ * ncm_vector_clear:
+ * @cv: a #NcmVector
+ *
+ * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
+ * all memory allocated by @cv is released. The pointer is set to NULL.
+ *
+ */
+void
+ncm_vector_clear (NcmVector **cv)
+{
+  g_clear_object (cv);
+}
+
 /**
  * ncm_vector_dup:
  * @cv: a constant #NcmVector
@@ -372,7 +522,7 @@ ncm_vector_const_new_variant (GVariant *var)
 NcmVector *
 ncm_vector_dup (const NcmVector *cv)
 {
-  NcmVector *cv_cp = ncm_vector_new (ncm_vector_len(cv));
+  NcmVector *cv_cp = ncm_vector_new (ncm_vector_len (cv));
   gsl_vector_memcpy (ncm_vector_gsl (cv_cp), ncm_vector_const_gsl (cv));
   return cv_cp;
 }
@@ -449,19 +599,20 @@ ncm_vector_get_subvector (NcmVector *cv, const gsize k, const gsize size)
 NcmVector *
 ncm_vector_get_subvector_stride (NcmVector *cv, const gsize k, const gsize size, const gsize stride)
 {
-  NcmVector *scv  = ncm_vector_new_data_static (ncm_vector_data (cv), size, stride);
-  const gsize len = ncm_vector_len (cv);
-
   g_assert_cmpuint (size,   >, 0);
   g_assert_cmpuint (stride, >, 0);
-  
-  g_assert_cmpuint ((size - 1) * stride + k, <, len);
+	{
+		NcmVector *scv  = ncm_vector_new_data_static (ncm_vector_ptr (cv, k), size, stride);
+		const gsize len = ncm_vector_len (cv);
 
-  scv->type  = NCM_VECTOR_DERIVED;
-  scv->pdata = ncm_vector_ref (cv);
-  scv->pfree = (GDestroyNotify) &ncm_vector_free;
+		g_assert_cmpuint ((size - 1) * stride + k, <, len);
 
-  return scv;
+		scv->type  = NCM_VECTOR_DERIVED;
+		scv->pdata = ncm_vector_ref (cv);
+		scv->pfree = (GDestroyNotify) &ncm_vector_free;
+
+		return scv;
+	}
 }
 
 /**
@@ -930,6 +1081,15 @@ ncm_vector_log_vals_func (const NcmVector *v, const gchar *prestr, const gchar *
  * Gets the minimum/maximum value of the vector components.
  *
  */
+/**
+ * ncm_vector_is_finite:
+ * @cv: a @NcmVector.
+ *
+ * Tests all entries, if one or more are not finite return FALSE.
+ * Otherwise returns TRUE;
+ * 
+ * Returns: whether all components of @cv are finite.
+ */
 
 /**
  * ncm_vector_get_absminmax:
@@ -1134,269 +1294,3 @@ ncm_vector_reciprocal (NcmVector *cv)
  *
  * Returns: the sum of the vector @cv components.
  */
-
-
-static void
-_ncm_vector_dispose (GObject *object)
-{
-  NcmVector *cv = NCM_VECTOR (object);
-
-  if (cv->pdata != NULL)
-  {
-    g_assert (cv->pfree != NULL);
-    cv->pfree (cv->pdata);
-    cv->pdata = NULL;
-    cv->pfree = NULL;
-  }
-
-  /* Chain up : end */
-  G_OBJECT_CLASS (ncm_vector_parent_class)->dispose (object);
-}
-
-static void
-_ncm_vector_finalize (GObject *object)
-{
-  NcmVector *cv = NCM_VECTOR (object);
-  switch (cv->type)
-  {
-    case NCM_VECTOR_SLICE:
-      g_slice_free1 (sizeof(gdouble) * ncm_vector_len (cv) * ncm_vector_stride (cv), ncm_vector_data (cv));
-      break;
-    case NCM_VECTOR_ARRAY:
-    case NCM_VECTOR_MALLOC:
-    case NCM_VECTOR_GSL_VECTOR:
-    case NCM_VECTOR_DERIVED:
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-  }
-  cv->vv.vector.data = NULL;
-
-  /* Chain up : end */
-  G_OBJECT_CLASS (ncm_vector_parent_class)->finalize (object);
-}
-
-/**
- * ncm_vector_free:
- * @cv: a #NcmVector
- *
- * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
- * all memory allocated by @cv is released.
- *
- */
-void
-ncm_vector_free (NcmVector *cv)
-{
-  g_object_unref (cv);
-}
-
-/**
- * ncm_vector_clear:
- * @cv: a #NcmVector
- *
- * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
- * all memory allocated by @cv is released. The pointer is set to NULL.
- *
- */
-void
-ncm_vector_clear (NcmVector **cv)
-{
-  g_clear_object (cv);
-}
-
-/**
- * ncm_vector_const_free:
- * @cv: a constant #NcmVector
- *
- * Atomically decrements the reference count of @cv by one. If the reference count drops to 0,
- * all memory allocated by @cv is released.
- *
- */
-void
-ncm_vector_const_free (const NcmVector *cv)
-{
-  ncm_vector_free (NCM_VECTOR (cv));
-}
-
-static struct _generic_N_Vector_Ops _ncm_ops;
-
-/**
- * ncm_vector_nvector: (skip)
- * @cv: a #NcmVector
- *
- * FIXME
- *
- * Returns: FIXME
- */
-N_Vector
-ncm_vector_nvector (NcmVector *cv)
-{
-  struct _generic_N_Vector *nv = g_new0 (struct _generic_N_Vector, 1);
-  if (cv != NULL)
-  {
-    if (ncm_vector_stride (cv) != 1)
-      g_error ("ncm_vector_nvector: cannot convert vector to N_Vector, stride must be 1.");
-    nv->content = ncm_vector_ref (cv);
-  }
-  nv->ops = &_ncm_ops;
-  return nv;
-}
-
-static N_Vector
-_ncm_nvclone (N_Vector nv)
-{
-  NcmVector *v = ncm_vector_new (ncm_vector_len (NCM_N2VECTOR (nv)));
-  N_Vector nnv = ncm_vector_nvector (v);
-  ncm_vector_free (v);
-  return nnv;
-}
-
-static N_Vector
-_ncm_nvcloneempty (N_Vector nv)
-{
-  NCM_UNUSED (nv);
-  return ncm_vector_nvector (NULL);
-}
-
-static void
-_ncm_nvspace (N_Vector nv, glong *lrw, glong *liw)
-{
-  *lrw = ncm_vector_len (NCM_N2VECTOR (nv));
-  *liw = (sizeof (NcmVector) % 4 == 0) ? sizeof (NcmVector) / 4 : sizeof (NcmVector) / 4 + 1;
-}
-
-static realtype *
-_ncm_nvgetarraypointer (N_Vector nv)
-{
-  return ncm_vector_data (NCM_N2VECTOR (nv));
-}
-
-static void
-_ncm_nvsetarraypointer (realtype *data, N_Vector nv)
-{
-  NcmVector *v = ncm_vector_new_data_malloc (data, ncm_vector_len (NCM_N2VECTOR (nv)), 1);
-  ncm_vector_free (NCM_N2VECTOR (nv));
-  nv->content = v;
-}
-
-static void
-_ncm_nvlinearsum (realtype a, N_Vector x, realtype b, N_Vector y, N_Vector z)
-{
-  NCM_UNUSED (a);
-  NCM_UNUSED (x);
-  NCM_UNUSED (b);
-  NCM_UNUSED (y);
-  NCM_UNUSED (z);
-}
-
-static void
-_ncm_nvconst (realtype a, N_Vector nv)
-{
-  gsl_vector_set_all (ncm_vector_gsl (NCM_N2VECTOR(nv)), a);
-}
-
-static void
-_ncm_vector_nvector_free (N_Vector nv)
-{
-  if (NCM_N2VECTOR (nv) != NULL)
-    ncm_vector_free (NCM_N2VECTOR (nv));
-  g_free (nv);
-}
-
-static struct _generic_N_Vector_Ops _ncm_ops =
-{
-#if defined (HAVE_SUNDIALS_2_7_0) || (HAVE_SUNDIALS_MAJOR == 3)
-  NULL,
-#endif /* HAVE_SUNDIALS_2_7_0 */
-  &_ncm_nvclone,
-  &_ncm_nvcloneempty,
-  &_ncm_vector_nvector_free,
-  &_ncm_nvspace,
-  &_ncm_nvgetarraypointer,
-  &_ncm_nvsetarraypointer,
-  &_ncm_nvlinearsum,
-  &_ncm_nvconst,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-};
-
-static void
-ncm_vector_init (NcmVector *v)
-{
-  v->pdata = NULL;
-  v->pfree = NULL;
-  v->type = 0;
-  memset (&v->vv, 0, sizeof (gsl_vector_view));
-}
-
-static void
-_ncm_vector_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
-{
-  NcmVector *v = NCM_VECTOR (object);
-  g_return_if_fail (NCM_IS_VECTOR (object));
-
-  switch (prop_id)
-  {
-    case PROP_VALS:
-    {
-      GVariant *var = ncm_vector_get_variant (v);
-      g_value_take_variant (value, var);
-      break;
-    }
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-_ncm_vector_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
-{
-  NcmVector *v = NCM_VECTOR (object);
-  g_return_if_fail (NCM_IS_VECTOR (object));
-
-  switch (prop_id)
-  {
-    case PROP_VALS:
-    {
-      GVariant *var = g_value_get_variant (value);
-      ncm_vector_set_from_variant (v, var);
-      break;
-    }
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-ncm_vector_class_init (NcmVectorClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  object_class->set_property = &_ncm_vector_set_property;
-  object_class->get_property = &_ncm_vector_get_property;
-  object_class->dispose = &_ncm_vector_dispose;
-  object_class->finalize = &_ncm_vector_finalize;
-
-  g_object_class_install_property (object_class, PROP_VALS,
-                                   g_param_spec_variant ("values", NULL, "values",
-                                                         G_VARIANT_TYPE_ARRAY, NULL,
-                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-
-}
