@@ -41,8 +41,6 @@
 #include "math/ncm_spline_cubic_notaknot.h"
 #include "model/nc_hicosmo_gcg.h"
 
-G_DEFINE_TYPE (NcHICosmoGCG, nc_hicosmo_gcg, NC_TYPE_HICOSMO);
-
 struct _NcHICosmoGCGPrivate
 {
   NcmSpline2d *BBN_spline2d;
@@ -54,6 +52,8 @@ struct _NcHICosmoGCGPrivate
   gdouble zmax;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (NcHICosmoGCG, nc_hicosmo_gcg, NC_TYPE_HICOSMO);
+
 enum
 {
   PROP_0,
@@ -235,8 +235,6 @@ nc_hicosmo_gcg_class_init (NcHICosmoGCGClass *klass)
   NcHICosmoClass *parent_class = NC_HICOSMO_CLASS (klass);
   NcmModelClass *model_class   = NCM_MODEL_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (NcHICosmoGCGPrivate));
-
   object_class->constructed = &_nc_hicosmo_gcg_constructed;
   object_class->dispose     = &_nc_hicosmo_gcg_dispose;
   object_class->finalize    = &_nc_hicosmo_gcg_finalize;
