@@ -41,8 +41,6 @@
 #include "math/ncm_spline_cubic_notaknot.h"
 #include "model/nc_hicosmo_idem2.h"
 
-G_DEFINE_TYPE (NcHICosmoIDEM2, nc_hicosmo_idem2, NC_TYPE_HICOSMO);
-
 struct _NcHICosmoIDEM2Private
 {
   NcmSpline2d *BBN_spline2d;
@@ -54,6 +52,8 @@ struct _NcHICosmoIDEM2Private
   gdouble zmax;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (NcHICosmoIDEM2, nc_hicosmo_idem2, NC_TYPE_HICOSMO);
+
 enum
 {
   PROP_0,
@@ -235,8 +235,6 @@ nc_hicosmo_idem2_class_init (NcHICosmoIDEM2Class *klass)
   NcHICosmoClass *parent_class = NC_HICOSMO_CLASS (klass);
   NcmModelClass *model_class   = NCM_MODEL_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (NcHICosmoIDEM2Private));
-
   object_class->constructed = &_nc_hicosmo_idem2_constructed;
   object_class->dispose     = &_nc_hicosmo_idem2_dispose;
   object_class->finalize    = &_nc_hicosmo_idem2_finalize;
