@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <numcosmo/build_cfg.h>
+#include <numcosmo/nc_hicosmo.h>
 #include <numcosmo/math/ncm_c.h>
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/math/ncm_mset_func.h>
@@ -43,19 +44,20 @@ G_BEGIN_DECLS
 #define NC_HIPRIM_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NC_TYPE_HIPRIM, NcHIPrimClass))
 
 typedef struct _NcHIPrimClass NcHIPrimClass;
-typedef struct _NcHIPrim NcHIPrim;
+/*typedef struct _NcHIPrim NcHIPrim;*/ /* already defined in nc_hicosmo.h! */
 
 /**
  * NcHIPrimImpl:
  * @NC_HIPRIM_IMPL_lnSA_powspec_lnk: Logarithm of the Scalar Adiabatic power spectrum as a function of $\ln(k\mathrm{Mpc})$
  * @NC_HIPRIM_IMPL_lnT_powspec_lnk: Logarithm of the Tensor power spectrum as a function of $\ln(k\mathrm{Mpc})$
  *
- * FIXME
+ * Methods to be implementd by every primordial model.
  */
 typedef enum _NcHIPrimImpl
 {
   NC_HIPRIM_IMPL_lnSA_powspec_lnk = 1 << 0,
-  NC_HIPRIM_IMPL_lnT_powspec_lnk  = 1 << 1, /*< private >*/
+  NC_HIPRIM_IMPL_lnT_powspec_lnk  = 1 << 1, 
+  /* < private > */
   NC_HIPRIM_IMPL_LAST             = 1 << 2, /*< skip >*/
 } NcHIPrimImpl;
 
