@@ -48,7 +48,7 @@
 #include "nc_distance.h"
 
 #include "math/integral.h"
-#include "math/memory_pool.h"
+#include "math/ncm_memory_pool.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
@@ -89,7 +89,7 @@ enum
   PROP_PKEQUAL,
 };
 
-G_DEFINE_TYPE (NcPowspecMNLHaloFit, nc_powspec_mnl_halofit, NC_TYPE_POWSPEC_MNL);
+G_DEFINE_TYPE_WITH_PRIVATE (NcPowspecMNLHaloFit, nc_powspec_mnl_halofit, NC_TYPE_POWSPEC_MNL);
 
 static void
 nc_powspec_mnl_halofit_init (NcPowspecMNLHaloFit* pshf)
@@ -108,8 +108,8 @@ nc_powspec_mnl_halofit_init (NcPowspecMNLHaloFit* pshf)
 	pshf->priv->linear_scale_solver = gsl_root_fdfsolver_alloc (gsl_root_fdfsolver_steffenson);
 	pshf->priv->znl_solver          = gsl_root_fsolver_alloc (gsl_root_fsolver_brent);
 
-	pshf->priv->z           = HUGE_VAL;
-  pshf->priv->pkequal     = FALSE;
+	pshf->priv->z        = HUGE_VAL;
+  pshf->priv->pkequal  = FALSE;
   pshf->priv->cpl      = NULL;
   pshf->priv->cpl_dist = NULL;
 }
@@ -235,8 +235,6 @@ nc_powspec_mnl_halofit_class_init (NcPowspecMNLHaloFitClass* klass)
 	object_class->dispose = &_nc_powspec_mnl_halofit_dispose;
 	object_class->finalize = &_nc_powspec_mnl_halofit_finalize;
 
-	g_type_class_add_private (klass, sizeof (NcPowspecMNLHaloFitPrivate));
-
 	g_object_class_install_property (object_class,
 	                                 PROP_PSML,
 	                                 g_param_spec_object ("power-spec",
@@ -422,8 +420,8 @@ _nc_powspec_mnl_halofit_linear_scale (NcPowspecMNLHaloFit* pshf, NcHICosmo* cosm
 		F.function = &_nc_powspec_mnl_halofit_varm1;
 		F.params = &vps;
 
-		gdouble lnRlo = -z / 2. - 10.;
-		gdouble lnRup = -z / 2. + 10.;
+		gdouble lnRlo = -z / 2.0 - 10.0;
+		gdouble lnRup = -z / 2.0 + 10.0;
 
 		gsl_root_fsolver_set (s, &F, lnRlo, lnRup);
 
