@@ -47,7 +47,12 @@
 #include "math/ncm_spline_cubic_notaknot.h"
 
 #ifndef NUMCOSMO_GIR_SCAN
+#include <cvode/cvode.h>
+#include <sundials/sundials_matrix.h>
+#include <sunmatrix/sunmatrix_dense.h>
+#include <sunlinsol/sunlinsol_dense.h>
 #include <nvector/nvector_serial.h>
+#define SUN_DENSE_ACCESS SM_ELEMENT_D
 #endif /* NUMCOSMO_GIR_SCAN */
 
 enum
@@ -59,59 +64,86 @@ enum
   PROP_DIST,
   PROP_RELTOL,
   PROP_ABSTOL,
-  PROP_TTYPE,
 };
 
-G_DEFINE_TYPE (NcScalefactor, nc_scalefactor, G_TYPE_OBJECT);
+struct _NcScalefactorPrivate
+{
+  NcDistance *dist;
+  NcmModelCtrl *ctrl;
+  NcmSpline *a_eta;
+  NcmSpline *eta_a;
+  NcmSpline *t_eta;
+  NcmSpline *eta_t;
+  gdouble a0;
+  gdouble zf;
+  gdouble eta_i;
+  gdouble eta_f;
+  gdouble t_i;
+  gboolean sets_conf_norm;
+  gboolean spline_init;
+  gboolean cvode_init;
+  gboolean quad_init;
+  gpointer cvode;
+  gdouble reltol;
+  gdouble abstol;
+  N_Vector y;
+  SUNMatrix A;
+  SUNLinearSolver LS;
+};
+
+G_DEFINE_TYPE_WITH_PRIVATE (NcScalefactor, nc_scalefactor, G_TYPE_OBJECT);
 
 static void
 nc_scalefactor_init (NcScalefactor *a)
 {
-  a->ttype       = 0;
-	a->a_eta       = ncm_spline_cubic_notaknot_new ();
-	a->eta_a       = ncm_spline_cubic_notaknot_new ();
-	a->t_eta       = ncm_spline_cubic_notaknot_new ();
-	a->eta_t       = ncm_spline_cubic_notaknot_new ();
-  a->spline_init = FALSE;
-	a->dist        = NULL;
-	a->ctrl        = ncm_model_ctrl_new (NULL);
-	a->zf          = 0.0;
-	a->a0          = 0.0;
+  NcScalefactorPrivate * const self = a->priv = G_TYPE_INSTANCE_GET_PRIVATE (a, NC_TYPE_SCALEFACTOR, NcScalefactorPrivate);
 
-  a->sets_conf_norm = FALSE;
+	self->a_eta       = ncm_spline_cubic_notaknot_new ();
+	self->eta_a       = ncm_spline_cubic_notaknot_new ();
+	self->t_eta       = ncm_spline_cubic_notaknot_new ();
+	self->eta_t       = ncm_spline_cubic_notaknot_new ();
+  self->spline_init = FALSE;
+	self->dist        = NULL;
+	self->ctrl        = ncm_model_ctrl_new (NULL);
 
-  a->cvode       = CVodeCreate (CV_BDF, CV_NEWTON);
-  NCM_CVODE_CHECK ((void *)a->cvode, "CVodeCreate", 0, );
-  a->cvode_init  = FALSE;
-  a->quad_init   = FALSE;
+  self->a0          = 0.0;
+	self->zf          = 0.0;
+	self->eta_i       = 0.0;
+	self->eta_f       = 0.0;
+	self->t_i         = 0.0;
 
-  a->reltol      = 0.0;
-  a->abstol      = 0.0;
+  self->sets_conf_norm = FALSE;
 
-  a->y           = N_VNew_Serial (1);
-  a->yQ          = N_VNew_Serial (1);
-#if HAVE_SUNDIALS_MAJOR == 3
-  a->A           = SUNDenseMatrix (1, 1);
-  a->LS          = SUNDenseLinearSolver (a->y, a->A);
+  self->cvode       = CVodeCreate (CV_BDF);
+  NCM_CVODE_CHECK ((void *)self->cvode, "CVodeCreate", 0, );
+  self->cvode_init  = FALSE;
+  self->quad_init   = FALSE;
+
+  self->reltol      = 0.0;
+  self->abstol      = 0.0;
+
+  self->y           = N_VNew_Serial (2);
+
+  self->A           = SUNDenseMatrix (2, 2);
+  self->LS          = SUNDenseLinearSolver (self->y, self->A);
   
-  NCM_CVODE_CHECK ((gpointer)a->A, "SUNDenseMatrix", 0, );
-  NCM_CVODE_CHECK ((gpointer)a->LS, "SUNDenseLinearSolver", 0, );
-#endif
-
+  NCM_CVODE_CHECK ((gpointer)self->A, "SUNDenseMatrix", 0, );
+  NCM_CVODE_CHECK ((gpointer)self->LS, "SUNDenseLinearSolver", 0, );
 }
 
 static void
 nc_scalefactor_dispose (GObject *object)
 {
   NcScalefactor *a = NC_SCALEFACTOR (object);
+  NcScalefactorPrivate * const self = a->priv;
   
-  ncm_spline_clear (&a->a_eta);
-  ncm_spline_clear (&a->eta_a);
-  ncm_spline_clear (&a->t_eta);
-  ncm_spline_clear (&a->eta_t);
+  ncm_spline_clear (&self->a_eta);
+  ncm_spline_clear (&self->eta_a);
+  ncm_spline_clear (&self->t_eta);
+  ncm_spline_clear (&self->eta_t);
 
-	ncm_model_ctrl_clear (&a->ctrl);
-	nc_distance_clear (&a->dist);
+	ncm_model_ctrl_clear (&self->ctrl);
+	nc_distance_clear (&self->dist);
 
   /* Chain up : end */
   G_OBJECT_CLASS (nc_scalefactor_parent_class)->dispose (object);
@@ -121,23 +153,21 @@ static void
 nc_scalefactor_finalize (GObject *object)
 {
   NcScalefactor *a = NC_SCALEFACTOR (object);
+  NcScalefactorPrivate * const self = a->priv;
   
-  CVodeFree (&a->cvode);
-  N_VDestroy (a->y);
-  N_VDestroy (a->yQ);
+  CVodeFree (&self->cvode);
+  N_VDestroy (self->y);
 
-#if HAVE_SUNDIALS_MAJOR == 3
-  if (a->A != NULL)
+  if (self->A != NULL)
   {
-    SUNMatDestroy (a->A);
-    a->A = NULL;
+    SUNMatDestroy (self->A);
+    self->A = NULL;
   }
-  if (a->LS != NULL)
+  if (self->LS != NULL)
   {
-    SUNLinSolFree (a->LS);
-    a->LS = NULL;
+    SUNLinSolFree (self->LS);
+    self->LS = NULL;
   }
-#endif
 
   /* Chain up : end */
   G_OBJECT_CLASS (nc_scalefactor_parent_class)->finalize (object);
@@ -147,6 +177,7 @@ static void
 nc_scalefactor_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcScalefactor *a = NC_SCALEFACTOR (object);
+  NcScalefactorPrivate * const self = a->priv;
   g_return_if_fail (NC_IS_SCALEFACTOR (object));
 
   switch (prop_id)
@@ -161,7 +192,7 @@ nc_scalefactor_set_property (GObject *object, guint prop_id, const GValue *value
       nc_scalefactor_set_a0_conformal_normal (a, g_value_get_boolean (value));
       break;
     case PROP_DIST:
-      a->dist = g_value_dup_object (value);
+      self->dist = g_value_dup_object (value);
       break;
     case PROP_RELTOL:
       nc_scalefactor_set_reltol (a, g_value_get_double (value));
@@ -169,9 +200,6 @@ nc_scalefactor_set_property (GObject *object, guint prop_id, const GValue *value
     case PROP_ABSTOL:
       nc_scalefactor_set_abstol (a, g_value_get_double (value));
       break;
-    case PROP_TTYPE:
-      nc_scalefactor_set_time_type (a, g_value_get_flags (value));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -182,6 +210,7 @@ static void
 nc_scalefactor_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcScalefactor *a = NC_SCALEFACTOR (object);
+  NcScalefactorPrivate * const self = a->priv;
   g_return_if_fail (NC_IS_SCALEFACTOR (object));
 
   switch (prop_id)
@@ -193,10 +222,10 @@ nc_scalefactor_get_property (GObject *object, guint prop_id, GValue *value, GPar
       g_value_set_double (value, nc_scalefactor_get_a0 (a));
       break;
     case PROP_A0_CONF_NORM:
-      g_value_set_boolean (value, a->sets_conf_norm);
+      g_value_set_boolean (value, self->sets_conf_norm);
       break;
     case PROP_DIST:
-      g_value_set_object (value, a->dist);
+      g_value_set_object (value, self->dist);
       break;
     case PROP_RELTOL:
       g_value_set_double (value, nc_scalefactor_get_reltol (a));
@@ -204,9 +233,6 @@ nc_scalefactor_get_property (GObject *object, guint prop_id, GValue *value, GPar
     case PROP_ABSTOL:
       g_value_set_double (value, nc_scalefactor_get_abstol (a));
       break;
-    case PROP_TTYPE:
-      g_value_set_flags (value, nc_scalefactor_get_time_type (a));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -265,29 +291,13 @@ nc_scalefactor_class_init (NcScalefactorClass *klass)
                                                         "Absolute tolerance",
                                                         0.0, G_MAXDOUBLE, NC_SCALEFACTOR_DEFAULT_ABSTOL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  g_object_class_install_property (object_class,
-                                   PROP_TTYPE,
-                                   g_param_spec_flags ("time-type",
-                                                        NULL,
-                                                        "Different time type to be integrated",
-                                                        NC_TYPE_SCALEFACTOR_TIME_TYPE, 0,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
 static gint dz_deta_f (realtype t, N_Vector y, N_Vector ydot, gpointer f_data);
-
-
-#if HAVE_SUNDIALS_MAJOR == 2
-static gint dz_deta_J (_NCM_SUNDIALS_INT_TYPE N, realtype lambda, N_Vector y, N_Vector fy, DlsMat J, gpointer jac_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
-#elif HAVE_SUNDIALS_MAJOR == 3
 static gint dz_deta_J (realtype lambda, N_Vector y, N_Vector fy, SUNMatrix J, void *jac_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
-#endif
-
-static gint dt_deta (realtype eta, N_Vector y, N_Vector yQdot, gpointer user_data);
 
 /**
  * nc_scalefactor_new:
- * @ttype: a #NcScalefactorTimeType
  * @zf: maximum redshift $z_f$
  * @dist: a #NcDistance
  *
@@ -297,16 +307,20 @@ static gint dt_deta (realtype eta, N_Vector y, N_Vector yQdot, gpointer user_dat
  * Returns: (transfer full): a new #NcScalefactor.
  */
 NcScalefactor *
-nc_scalefactor_new (NcScalefactorTimeType ttype, gdouble zf, NcDistance *dist)
+nc_scalefactor_new (gdouble zf, NcDistance *dist)
 {
   NcScalefactor *a;
+
   if (dist == NULL)
     dist = nc_distance_new (1.0);
+  else
+    dist = nc_distance_ref (dist);
+  
   a = g_object_new (NC_TYPE_SCALEFACTOR,
-                                     "time-type", ttype,
-                                     "zf", zf,
-                                     "dist", dist,
-                                     NULL);
+                    "zf", zf,
+                    "dist", dist,
+                    NULL);
+
   nc_distance_free (dist);
   return a;
 }
@@ -366,11 +380,12 @@ static void _nc_scalefactor_calc_spline (NcScalefactor *a, NcHICosmo *cosmo);
 void
 nc_scalefactor_prepare (NcScalefactor *a, NcHICosmo *cosmo)
 {
-  nc_distance_prepare_if_needed (a->dist, cosmo);
+  NcScalefactorPrivate * const self = a->priv;
+  nc_distance_prepare_if_needed (self->dist, cosmo);
   
 	_nc_scalefactor_init (a, cosmo);
 	_nc_scalefactor_calc_spline (a, cosmo);
-	ncm_model_ctrl_update (a->ctrl, NCM_MODEL (cosmo));
+	ncm_model_ctrl_update (self->ctrl, NCM_MODEL (cosmo));
 }
 
 /**
@@ -384,7 +399,8 @@ nc_scalefactor_prepare (NcScalefactor *a, NcHICosmo *cosmo)
 void
 nc_scalefactor_prepare_if_needed (NcScalefactor *a, NcHICosmo *cosmo)
 {
-	if (ncm_model_ctrl_update (a->ctrl, NCM_MODEL (cosmo)))
+  NcScalefactorPrivate * const self = a->priv;
+	if (ncm_model_ctrl_update (self->ctrl, NCM_MODEL (cosmo)))
 		nc_scalefactor_prepare (a, cosmo);
 }
 
@@ -399,10 +415,11 @@ nc_scalefactor_prepare_if_needed (NcScalefactor *a, NcHICosmo *cosmo)
 void 
 nc_scalefactor_set_zf (NcScalefactor *a, const gdouble zf)
 {
-  if (a->zf != zf)
+  NcScalefactorPrivate * const self = a->priv;
+  if (self->zf != zf)
   {
-    a->zf = zf;
-    ncm_model_ctrl_force_update (a->ctrl);
+    self->zf = zf;
+    ncm_model_ctrl_force_update (self->ctrl);
   }
 }
 
@@ -417,10 +434,11 @@ nc_scalefactor_set_zf (NcScalefactor *a, const gdouble zf)
 void 
 nc_scalefactor_set_a0 (NcScalefactor *a, const gdouble a0)
 {
-  if (a->a0 != a0)
+  NcScalefactorPrivate * const self = a->priv;
+  if (self->a0 != a0)
   {
-    a->a0 = a0;
-    ncm_model_ctrl_force_update (a->ctrl);
+    self->a0 = a0;
+    ncm_model_ctrl_force_update (self->ctrl);
   }
 }
 
@@ -435,10 +453,11 @@ nc_scalefactor_set_a0 (NcScalefactor *a, const gdouble a0)
 void 
 nc_scalefactor_set_reltol (NcScalefactor *a, const gdouble reltol)
 {
-  if (a->reltol != reltol)
+  NcScalefactorPrivate * const self = a->priv;
+  if (self->reltol != reltol)
   {
-    a->reltol = reltol;
-    ncm_model_ctrl_force_update (a->ctrl);
+    self->reltol = reltol;
+    ncm_model_ctrl_force_update (self->ctrl);
   }
 }
 
@@ -453,28 +472,11 @@ nc_scalefactor_set_reltol (NcScalefactor *a, const gdouble reltol)
 void
 nc_scalefactor_set_abstol (NcScalefactor *a, const gdouble abstol)
 {
-  if (a->abstol != abstol)
+  NcScalefactorPrivate * const self = a->priv;
+  if (self->abstol != abstol)
   {
-    a->abstol = abstol;
-    ncm_model_ctrl_force_update (a->ctrl);
-  }
-}
-
-/**
- * nc_scalefactor_set_time_type:
- * @a: a #NcScalefactor
- * @ttype: a #NcScalefactorTimeType flag
- * 
- * Sets the which other time variables it should integrate.
- * 
- */
-void 
-nc_scalefactor_set_time_type (NcScalefactor *a, NcScalefactorTimeType ttype)
-{
-  if (a->ttype != ttype)
-  {
-    a->ttype = ttype;
-    ncm_model_ctrl_force_update (a->ctrl);
+    self->abstol = abstol;
+    ncm_model_ctrl_force_update (self->ctrl);
   }
 }
 
@@ -493,10 +495,11 @@ nc_scalefactor_set_time_type (NcScalefactor *a, NcScalefactorTimeType ttype)
 void 
 nc_scalefactor_set_a0_conformal_normal (NcScalefactor *a, gboolean enable) 
 {
-  if ((!a->sets_conf_norm && enable) || (a->sets_conf_norm && !enable))
+  NcScalefactorPrivate * const self = a->priv;
+  if ((!self->sets_conf_norm && enable) || (self->sets_conf_norm && !enable))
   {
-    a->sets_conf_norm = enable;
-    ncm_model_ctrl_force_update (a->ctrl);
+    self->sets_conf_norm = enable;
+    ncm_model_ctrl_force_update (self->ctrl);
   }
 }
 
@@ -511,7 +514,8 @@ nc_scalefactor_set_a0_conformal_normal (NcScalefactor *a, gboolean enable)
 gdouble 
 nc_scalefactor_get_zf (NcScalefactor *a)
 {
-  return a->zf;
+  NcScalefactorPrivate * const self = a->priv;
+  return self->zf;
 }
 
 /**
@@ -525,7 +529,8 @@ nc_scalefactor_get_zf (NcScalefactor *a)
 gdouble 
 nc_scalefactor_get_a0 (NcScalefactor *a)
 {
-  return a->a0;
+  NcScalefactorPrivate * const self = a->priv;
+  return self->a0;
 }
 
 /**
@@ -539,7 +544,8 @@ nc_scalefactor_get_a0 (NcScalefactor *a)
 gdouble 
 nc_scalefactor_get_reltol (NcScalefactor *a)
 {
-  return a->reltol;
+  NcScalefactorPrivate * const self = a->priv;
+  return self->reltol;
 }
 
 /**
@@ -553,21 +559,8 @@ nc_scalefactor_get_reltol (NcScalefactor *a)
 gdouble 
 nc_scalefactor_get_abstol (NcScalefactor *a)
 {
-  return a->abstol;
-}
-
-/**
- * nc_scalefactor_get_time_type:
- * @a: a #NcScalefactor
- * 
- * Gets the current time type flag.
- * 
- * Returns: ttime.
- */
-NcScalefactorTimeType 
-nc_scalefactor_get_time_type (NcScalefactor *a)
-{
-  return a->ttype;
+  NcScalefactorPrivate * const self = a->priv;
+  return self->abstol;
 }
 
 static void nc_scalefactor_init_cvode (NcScalefactor *a, NcHICosmo *cosmo);
@@ -575,23 +568,26 @@ static void nc_scalefactor_init_cvode (NcScalefactor *a, NcHICosmo *cosmo);
 static void
 _nc_scalefactor_init (NcScalefactor *a, NcHICosmo *cosmo)
 {
+  NcScalefactorPrivate * const self = a->priv;
   const gdouble Omega_k0 = nc_hicosmo_Omega_k0 (cosmo);
 
-  NV_Ith_S (a->y, 0) = a->zf;
+  self->eta_i = nc_distance_conformal_time (self->dist, cosmo, self->zf);
+  self->eta_f = nc_distance_conformal_time (self->dist, cosmo, 0.0);
+  self->t_i   = nc_distance_cosmic_time (self->dist, cosmo, self->zf);
 
-  a->eta_i = nc_distance_conformal_time (a->dist, cosmo, a->zf);
-  a->eta_f = nc_distance_conformal_time (a->dist, cosmo, 0.0);
-
-  if (a->sets_conf_norm)
+  NV_Ith_S (self->y, 0) = self->zf;
+  NV_Ith_S (self->y, 1) = self->t_i;
+  
+  if (self->sets_conf_norm)
   {
     if (fabs (Omega_k0) < NC_SCALEFACTOR_OMEGA_K_ZERO)
     {
-      a->a0 = 1.0;
+      self->a0 = 1.0;
     }
     else
     {
       const gdouble RH_Mpc = nc_hicosmo_RH_Mpc (cosmo);
-      a->a0 = RH_Mpc / sqrt (fabs (Omega_k0));
+      self->a0 = RH_Mpc / sqrt (fabs (Omega_k0));
     }
   }
   nc_scalefactor_init_cvode (a, cosmo);
@@ -602,46 +598,38 @@ _nc_scalefactor_init (NcScalefactor *a, NcHICosmo *cosmo)
 static void
 nc_scalefactor_init_cvode (NcScalefactor *a, NcHICosmo *cosmo)
 {
+  NcScalefactorPrivate * const self = a->priv;
   gint flag;
 
-  if (a->cvode_init)
+  if (self->cvode_init)
   {
-    flag = CVodeReInit (a->cvode, a->eta_i, a->y);
+    flag = CVodeReInit (self->cvode, self->eta_i, self->y);
     NCM_CVODE_CHECK (&flag, "CVodeReInit", 1, );
   }
   else
   {
-    flag = CVodeInit (a->cvode, &dz_deta_f, a->eta_i, a->y);
+    flag = CVodeInit (self->cvode, &dz_deta_f, self->eta_i, self->y);
     NCM_CVODE_CHECK (&flag, "CVodeMalloc", 1, );
-    a->cvode_init = TRUE;
+    self->cvode_init = TRUE;
   }
 
-  flag = CVodeSStolerances (a->cvode, a->reltol, a->abstol);
+  flag = CVodeSStolerances (self->cvode, self->reltol, self->abstol);
   NCM_CVODE_CHECK (&flag, "CVodeSStolerances", 1, );
 
-  flag = CVodeSetStopTime(a->cvode, a->eta_f);
+  flag = CVodeSetStopTime(self->cvode, self->eta_f);
   NCM_CVODE_CHECK (&flag, "CVodeSetStopTime", 1, );
 
-  flag = CVodeSetUserData (a->cvode, cosmo);
+  flag = CVodeSetUserData (self->cvode, cosmo);
   NCM_CVODE_CHECK (&flag, "CVodeSetUserData", 1, );
 
-  flag = CVodeSetMaxNumSteps (a->cvode, 50000);
+  flag = CVodeSetMaxNumSteps (self->cvode, 50000);
   NCM_CVODE_CHECK (&flag, "CVodeSetMaxNumSteps", 1, );
 
+  flag = CVodeSetLinearSolver (self->cvode, self->LS, self->A);
+  NCM_CVODE_CHECK (&flag, "CVodeSetLinearSolver", 1, );
 
-#if HAVE_SUNDIALS_MAJOR == 2
-  flag = CVDense(a->cvode, 1);
-  NCM_CVODE_CHECK (&flag, "CVDense", 1, );
-
-  flag = CVDlsSetDenseJacFn (a->cvode, &dz_deta_J);
-  NCM_CVODE_CHECK (&flag, "CVDlsSetDenseJacFn", 1, );
-#elif HAVE_SUNDIALS_MAJOR == 3
-  flag = CVDlsSetLinearSolver (a->cvode, a->LS, a->A);
-  NCM_CVODE_CHECK (&flag, "CVDlsSetLinearSolver", 1, );
-
-  flag = CVDlsSetJacFn (a->cvode, &dz_deta_J);
-  NCM_CVODE_CHECK (&flag, "CVDlsSetJacFn", 1, );
-#endif
+  flag = CVodeSetJacFn (self->cvode, &dz_deta_J);
+  NCM_CVODE_CHECK (&flag, "CVodeSetJacFn", 1, );
   
   return;
 }
@@ -659,7 +647,8 @@ nc_scalefactor_init_cvode (NcScalefactor *a, NcHICosmo *cosmo)
 gdouble
 nc_scalefactor_eval_z_eta (NcScalefactor *a, const gdouble eta)
 {
-  return -ncm_spline_eval (a->a_eta, eta);
+  NcScalefactorPrivate * const self = a->priv;
+  return -ncm_spline_eval (self->a_eta, eta);
 }
 
 /**
@@ -675,7 +664,8 @@ nc_scalefactor_eval_z_eta (NcScalefactor *a, const gdouble eta)
 gdouble
 nc_scalefactor_eval_eta_z (NcScalefactor *a, const gdouble z)
 {
-  return ncm_spline_eval (a->eta_a, - z);
+  NcScalefactorPrivate * const self = a->priv;
+  return ncm_spline_eval (self->eta_a, - z);
 }
 
 /**
@@ -691,7 +681,8 @@ nc_scalefactor_eval_eta_z (NcScalefactor *a, const gdouble z)
 gdouble
 nc_scalefactor_eval_eta_x (NcScalefactor *a, const gdouble x)
 {
-  return ncm_spline_eval (a->eta_a, -(x - 1.0));
+  NcScalefactorPrivate * const self = a->priv;
+  return ncm_spline_eval (self->eta_a, -(x - 1.0));
 }
 
 /**
@@ -707,9 +698,10 @@ nc_scalefactor_eval_eta_x (NcScalefactor *a, const gdouble x)
 gdouble
 nc_scalefactor_eval_a_eta (NcScalefactor *a, const gdouble eta)
 {
+  NcScalefactorPrivate * const self = a->priv;
   gdouble mz;
-  mz = ncm_spline_eval (a->a_eta, eta);
-  return a->a0 / (1.0 - mz);
+  mz = ncm_spline_eval (self->a_eta, eta);
+  return self->a0 / (1.0 - mz);
 }
 
 /**
@@ -725,7 +717,8 @@ nc_scalefactor_eval_a_eta (NcScalefactor *a, const gdouble eta)
 gdouble 
 nc_scalefactor_eval_t_eta (NcScalefactor *a, const gdouble eta)
 {
-  return ncm_spline_eval (a->t_eta, eta);
+  NcScalefactorPrivate * const self = a->priv;
+  return ncm_spline_eval (self->t_eta, eta);
 }
 
 /**
@@ -741,21 +734,22 @@ nc_scalefactor_eval_t_eta (NcScalefactor *a, const gdouble eta)
 gdouble 
 nc_scalefactor_eval_eta_t (NcScalefactor *a, const gdouble t)
 {
-  return ncm_spline_eval (a->eta_t, t);
+  NcScalefactorPrivate * const self = a->priv;
+  return ncm_spline_eval (self->eta_t, t);
 }
 
 static void
 _nc_scalefactor_calc_spline (NcScalefactor *a, NcHICosmo *cosmo)
 {
+  NcScalefactorPrivate * const self = a->priv;
   GArray *eta_a, *mz_a, *t_a = NULL;
-  gdouble eta, mzi, last_eta;
+  gdouble mzi, last_eta;
   gint flag;
-  gboolean int_t = (a->ttype & NC_SCALEFACTOR_TIME_TYPE_COSMIC);
 
-	if (!ncm_spline_is_empty (a->a_eta))
+	if (!ncm_spline_is_empty (self->a_eta))
 	{
-		eta_a = ncm_vector_get_array (a->a_eta->xv);
-		mz_a  = ncm_vector_get_array (a->a_eta->yv);
+		eta_a = ncm_vector_get_array (self->a_eta->xv);
+		mz_a  = ncm_vector_get_array (self->a_eta->yv);
 		g_array_set_size (eta_a, 0);
 		g_array_set_size (mz_a, 0);
 	}
@@ -765,91 +759,44 @@ _nc_scalefactor_calc_spline (NcScalefactor *a, NcHICosmo *cosmo)
 		mz_a  = g_array_sized_new (FALSE, FALSE, sizeof(gdouble), 1000);
 	}
 
-  last_eta = a->eta_i;
-  mzi      = -a->zf;
-  g_array_append_val (eta_a, a->eta_i);
-  g_array_append_val (mz_a, mzi);
-
-  if (int_t)
+  if (!ncm_spline_is_empty (self->t_eta))
   {
-    const gdouble ti = nc_distance_cosmic_time (a->dist, cosmo, a->zf);
-    NV_Ith_S (a->yQ, 0) = ti;
-
-    if (a->quad_init)
-    {
-      flag = CVodeQuadReInit (a->cvode, a->yQ);
-      NCM_CVODE_CHECK (&flag, "_nc_scalefactor_calc_spline[CVodeQuadReInit]", 1, );
-    }
-    else
-    {
-      flag = CVodeQuadInit (a->cvode, &dt_deta, a->yQ);
-      NCM_CVODE_CHECK (&flag, "_nc_scalefactor_calc_spline[CVodeQuadInit]", 1, );
-      a->quad_init = TRUE;
-    }
-
-    if (!ncm_spline_is_empty (a->t_eta))
-    {
-      t_a = ncm_vector_get_array (a->t_eta->yv);
-      g_array_set_size (t_a, 0);
-    }
-    else
-      t_a = g_array_sized_new (FALSE, FALSE, sizeof(gdouble), 1000);
-
-    g_array_append_val (t_a, ti);
-  }
-  else if (a->quad_init)
-  {
-    CVodeQuadFree (a->cvode);
-    a->quad_init = FALSE;
-  }
-  
-  if (int_t)
-  {
-    while (TRUE)
-    {
-      gdouble eta_quad, t;
-      
-      flag = CVode (a->cvode, a->eta_f, a->y, &eta, CV_ONE_STEP);
-      NCM_CVODE_CHECK (&flag, "_nc_scalefactor_calc_spline[CVode]", 1, );
-      flag = CVodeGetQuad (a->cvode, &eta_quad, a->yQ);
-
-      if ((eta - last_eta) / last_eta > NC_SCALEFACTOR_MIN_ETA_STEP)
-      {
-        mzi = - NV_Ith_S (a->y, 0);
-        t   = NV_Ith_S (a->yQ, 0);
-
-        g_array_append_val (eta_a, eta);
-        g_array_append_val (mz_a, mzi);
-        g_array_append_val (t_a, t);
-        last_eta = eta;
-      }
-      
-      if (a->eta_f == eta)
-        break;
-    }
+    t_a = ncm_vector_get_array (self->t_eta->yv);
+    g_array_set_size (t_a, 0);
   }
   else
+    t_a = g_array_sized_new (FALSE, FALSE, sizeof(gdouble), 1000);
+
+  
+  last_eta = self->eta_i;
+  mzi      = -self->zf;
+  
+  g_array_append_val (eta_a, self->eta_i);
+  g_array_append_val (mz_a, mzi);
+  g_array_append_val (t_a, self->t_i);
+
+  while (TRUE)
   {
-    while (TRUE)
+    gdouble eta;
+
+    flag = CVode (self->cvode, self->eta_f, self->y, &eta, CV_ONE_STEP);
+    NCM_CVODE_CHECK (&flag, "_nc_scalefactor_calc_spline[CVode]", 1, );
+
+    if ((eta - last_eta) / last_eta > NC_SCALEFACTOR_MIN_ETA_STEP)
     {
-      flag = CVode (a->cvode, a->eta_f, a->y, &eta, CV_ONE_STEP);
-      NCM_CVODE_CHECK (&flag, "_nc_scalefactor_calc_spline[CVode]", 1, );
+      const gdouble mz = - NV_Ith_S (self->y, 0);
+      const gdouble t  = NV_Ith_S (self->y, 1);
 
-      if ((eta - last_eta) / last_eta > NC_SCALEFACTOR_MIN_ETA_STEP)
-      {
-        mzi = - NV_Ith_S (a->y, 0);
-
-        g_array_append_val (eta_a, eta);
-        g_array_append_val (mz_a, mzi);
-
-        last_eta = eta;
-      }
-      
-      if (a->eta_f == eta)
-        break;
+      g_array_append_val (eta_a, eta);
+      g_array_append_val (mz_a, mz);
+      g_array_append_val (t_a, t);
+      last_eta = eta;
     }
-  }
 
+    if (self->eta_f == eta)
+      break;
+  }
+  
 	if (fabs (g_array_index (mz_a, gdouble, mz_a->len - 1)) < 1e-10)
 	{
 		g_array_index (mz_a, gdouble, mz_a->len - 1) = 0.0;
@@ -858,17 +805,14 @@ _nc_scalefactor_calc_spline (NcScalefactor *a, NcHICosmo *cosmo)
 		g_error ("_nc_scalefactor_calc_spline today redshift must be zero not % 20.15g.",
 		         fabs (g_array_index (mz_a, gdouble, mz_a->len - 1)));
 
-	ncm_spline_set_array (a->a_eta, eta_a, mz_a, TRUE);
-	ncm_spline_set_array (a->eta_a, mz_a, eta_a, TRUE);
+	ncm_spline_set_array (self->a_eta, eta_a, mz_a, TRUE);
+	ncm_spline_set_array (self->eta_a, mz_a, eta_a, TRUE);
   g_array_unref (eta_a);
   g_array_unref (mz_a);
 
-  if (int_t)
-  {
-    ncm_spline_set_array (a->t_eta, eta_a, t_a, TRUE);
-    ncm_spline_set_array (a->eta_t, t_a, eta_a, TRUE);
-    g_array_unref (t_a);
-  }
+  ncm_spline_set_array (self->t_eta, eta_a, t_a, TRUE);
+  ncm_spline_set_array (self->eta_t, t_a, eta_a, TRUE);
+  g_array_unref (t_a);
   
   return;
 }
@@ -878,45 +822,28 @@ dz_deta_f (realtype t, N_Vector y, N_Vector ydot, gpointer f_data)
 {
   NcHICosmo *cosmo = NC_HICOSMO (f_data);
   const gdouble z = NV_Ith_S (y, 0);
+  const gdouble x = 1.0 + z;
   const gdouble E = nc_hicosmo_E (cosmo, z);
 
-  NCM_UNUSED (t);
-
   NV_Ith_S (ydot, 0) = - E;
+  NV_Ith_S (ydot, 1) = 1.0 / x;
   return 0;
 }
 
 static gint
-#if HAVE_SUNDIALS_MAJOR == 2
-dz_deta_J (_NCM_SUNDIALS_INT_TYPE N, realtype t, N_Vector y, N_Vector fy, DlsMat J, gpointer jac_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
-#elif HAVE_SUNDIALS_MAJOR == 3
 dz_deta_J (realtype t, N_Vector y, N_Vector fy, SUNMatrix J, void *jac_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
-#endif
 {
-  NcHICosmo *cosmo = NC_HICOSMO (jac_data);
-  const gdouble z = NV_Ith_S (y, 0);
-  const gdouble E = nc_hicosmo_E (cosmo, z);
+  NcHICosmo *cosmo     = NC_HICOSMO (jac_data);
+  const gdouble z      = NV_Ith_S (y, 0);
+  const gdouble x      = 1.0 + z;
+  const gdouble E      = nc_hicosmo_E (cosmo, z);
   const gdouble dE2_dz = nc_hicosmo_dE2_dz (cosmo, z);
-
-  NCM_UNUSED (t);
-  NCM_UNUSED (fy);
-  NCM_UNUSED (tmp1);
-  NCM_UNUSED (tmp2);
-  NCM_UNUSED (tmp3);
   
   SUN_DENSE_ACCESS (J, 0, 0) = - dE2_dz / (2.0 * E);
+  SUN_DENSE_ACCESS (J, 0, 1) = 0.0;
 
+  SUN_DENSE_ACCESS (J, 1, 0) = - 1.0 / (x * x);
+  SUN_DENSE_ACCESS (J, 1, 1) = 0.0;
+  
   return 0;
 }
-
-static gint 
-dt_deta (realtype eta, N_Vector y, N_Vector yQdot, gpointer user_data)
-{
-  const gdouble z = NV_Ith_S (y, 0);
-  const gdouble x = 1.0 + z;
-
-  NV_Ith_S (yQdot, 0) = 1.0 / x;
-
-  return 0;
-}
-
