@@ -30,23 +30,6 @@
 #include <numcosmo/build_cfg.h>
 #include <numcosmo/nc_distance.h>
 
-#ifndef NUMCOSMO_GIR_SCAN
-#include <cvodes/cvodes.h>
-#if HAVE_SUNDIALS_MAJOR == 2
-#include <cvodes/cvodes_dense.h>
-#define SUN_DENSE_ACCESS DENSE_ELEM
-#elif HAVE_SUNDIALS_MAJOR == 3
-#include <cvodes/cvodes_direct.h>
-#endif
-#if HAVE_SUNDIALS_MAJOR == 3
-#include <sundials/sundials_matrix.h>
-#include <sunmatrix/sunmatrix_dense.h>
-#include <sunlinsol/sunlinsol_dense.h>
-#define SUN_DENSE_ACCESS SM_ELEMENT_D
-#endif 
-#include <nvector/nvector_serial.h>
-#endif /* NUMCOSMO_GIR_SCAN */
-
 G_BEGIN_DECLS
 
 #define NC_TYPE_SCALEFACTOR             (nc_scalefactor_get_type ())
@@ -58,20 +41,7 @@ G_BEGIN_DECLS
 
 typedef struct _NcScalefactorClass NcScalefactorClass;
 typedef struct _NcScalefactor NcScalefactor;
-
-/**
- * NcScalefactorTimeType:
- * @NC_SCALEFACTOR_TIME_TYPE_COSMIC: Cosmic time $\mathrm{d}t = a\mathrm{d}\eta$.
- *
- * Controls which time variables are also integrated.
- * 
- */
-typedef enum _NcScalefactorTimeType
-{
-  NC_SCALEFACTOR_TIME_TYPE_COSMIC = 1 << 0, 
-  /* < private > */
-  NC_SCALEFACTOR_TIME_TYPE_ALL    = (1 << 1) - 1, /*< skip >*/  
-} NcScalefactorTimeType;
+typedef struct _NcScalefactorPrivate NcScalefactorPrivate;
 
 struct _NcScalefactorClass
 {
@@ -83,35 +53,12 @@ struct _NcScalefactor
 {
   /*< private >*/
   GObject parent_instance;
-  NcDistance *dist;
-  NcScalefactorTimeType ttype;
-  NcmModelCtrl *ctrl;
-  NcmSpline *a_eta;
-  NcmSpline *eta_a;
-  NcmSpline *t_eta;
-  NcmSpline *eta_t;
-  gdouble a0;
-  gdouble zf;
-  gdouble eta_i;
-  gdouble eta_f;
-  gboolean sets_conf_norm;
-  gboolean spline_init;
-  gboolean cvode_init;
-  gboolean quad_init;
-  gpointer cvode;
-  gdouble reltol;
-  gdouble abstol;
-  N_Vector y;
-  N_Vector yQ;
-#if HAVE_SUNDIALS_MAJOR == 3
-  SUNMatrix A;
-  SUNLinearSolver LS;
-#endif
+  NcScalefactorPrivate *priv;
 };
 
 GType nc_scalefactor_get_type (void) G_GNUC_CONST;
 
-NcScalefactor *nc_scalefactor_new (NcScalefactorTimeType ttype, const gdouble zf, NcDistance *dist);
+NcScalefactor *nc_scalefactor_new (const gdouble zf, NcDistance *dist);
 NcScalefactor *nc_scalefactor_ref (NcScalefactor *a);
 void nc_scalefactor_free (NcScalefactor *a);
 void nc_scalefactor_clear (NcScalefactor **a);
@@ -122,7 +69,6 @@ void nc_scalefactor_set_zf (NcScalefactor *a, const gdouble zf);
 void nc_scalefactor_set_a0 (NcScalefactor *a, const gdouble a0);
 void nc_scalefactor_set_reltol (NcScalefactor *a, const gdouble reltol);
 void nc_scalefactor_set_abstol (NcScalefactor *a, const gdouble abstol);
-void nc_scalefactor_set_time_type (NcScalefactor *a, NcScalefactorTimeType ttype);
 
 void nc_scalefactor_set_a0_conformal_normal (NcScalefactor *a, gboolean enable);
 
@@ -130,7 +76,6 @@ gdouble nc_scalefactor_get_zf (NcScalefactor *a);
 gdouble nc_scalefactor_get_a0 (NcScalefactor *a);
 gdouble nc_scalefactor_get_reltol (NcScalefactor *a);
 gdouble nc_scalefactor_get_abstol (NcScalefactor *a);
-NcScalefactorTimeType nc_scalefactor_get_time_type (NcScalefactor *a);
 
 gdouble nc_scalefactor_eval_z_eta (NcScalefactor *a, const gdouble eta);
 gdouble nc_scalefactor_eval_a_eta (NcScalefactor *a, const gdouble eta);
